package com.ulsanonline.triominoes;
/*
 * 02/22/2012 - v1.001 Initial release
 * 03/10/2012 - v2.0 - allow the user to play a solo game
 * 03/18/2012 - v3.1 - play against the Android computer
 * 09/30/2012 - v3.4 - improved multithreading for game startup.
 * 11/23/2012 - v3.5 - call finish() after starting game to clean up quicker. Close database after opening
 * 12/22/2012 - v3.9   - game_id: int -> long
 * 01/13/2012 - v4.0 - hide keyboard initially until needed
 * 09/29/2013 - v4.3 - threaded all calls to server
 * 
 */


import org.json.JSONException;


import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.Toast;

public class ChallengeUser extends Activity {

	private EditText mNameText;
	private ServerCommunication sComm; 
	private triDbAdapter mDbHelper;
	private String userName;
	private int user_id;
	private long game_id;
	private String OpponentName;
	private Context mContext;
	private Bundle bundle;
	private Intent intent;
	private Boolean startError;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.challenge_user);
		setTitle("Choose your Opponent");
		mNameText = (EditText)findViewById(R.id.username);
		mContext = this;
		sComm = new ServerCommunication(this);
		mDbHelper = new triDbAdapter(this.getApplicationContext() );
		
	}
	
	private void populateData() {
		// fetch local user name
		Cursor c = mDbHelper.fetchPlayer();
		if (c.moveToFirst()) {
		   int nameColumn = c.getColumnIndex(mDbHelper.KEY_NAME);
		   userName = c.getString(nameColumn); 
		   int idColumn  = c.getColumnIndex(mDbHelper.KEY_ID);
		   user_id = c.getInt(idColumn);
		}
		c.close();

	}

	
	// Will be called via the onClick attribute
	// of the buttons in main.xml
	public void onClick(View view) {
		
	    startError = false;
	    intent = new Intent(this, TriOminoesStartActivity.class);
		bundle = new Bundle();
		
		switch (view.getId()) {
			
		case R.id.solo:
		    // make a loner game
			doServerCommunication(R.id.solo, "Creating Solo Game...");
			
		    break;
		    
		case R.id.friend:
			OpponentName = mNameText.getText().toString();
        	if (OpponentName.isEmpty()) {
        		Toast.makeText(this, "You need a name here.", Toast.LENGTH_SHORT).show();
        	}
        	else {
				doServerCommunication(R.id.friend, "Notifying User of Challenge...");
        	}
			break;
			
		case R.id.random:
			OpponentName = "_random_player"; // keyword to tell server to pick from server Db
			doServerCommunication(R.id.random, "Finding random user...");

			break;
			
		case R.id.device:
			OpponentName = "ANDROID"; // keyword to tell server to play against the machine

			doServerCommunication(R.id.device, "Creating local game against ANDROID...");

			break;	
			
			
		}
		

	}
	
	private void doServerCommunication (final int buttonId, String messsage) {
		
		final ProgressDialog dialog = new ProgressDialog(mContext);

		dialog.setMessage(messsage);  
		dialog.show();
    	
        	new AsyncTask<Void,Void, Boolean>(){
                @Override
                protected void onPostExecute(Boolean startError) {
                	Log.d("ChUser", "AsyncTask has finished");
                	
                	dialog.dismiss();
                	// intent has been created and stuffed with it's bundle
            		// being the game if no errors encountered
                	if (!startError) {
            			startActivity(intent);
            			finish();
            		}
                	
                }

				@Override
				protected Boolean doInBackground(Void... params) {
					
					try {
						switch (buttonId) {
						case R.id.solo :
							try {
								game_id = sComm.sologame(userName, user_id);
								if (game_id > 0) {
									// pull down the remainder of the tiles and create local server 
								    sComm.gameLocalServer(game_id);
								    			    
							        bundle.putLong(mDbHelper.KEY_ID, game_id);
									intent.putExtras(bundle); 
						    	}
								else {
									startError = true;
								}
								
							} catch (JSONException e) {
								e.printStackTrace();
							}
							break;
							
						case R.id.friend :
							try {

   							     game_id = sComm.gameChallenge(userName, user_id, OpponentName);
   							     if (game_id <= 0) 
   			        		     {
   			        			    startError = true;
   			        			    // the only error we could get is non-existent name
   								    Toast.makeText(mContext, "Sorry, username:" + OpponentName +
   										" doesn't exist. Try another name.", Toast.LENGTH_SHORT).show();
   			        		     }
   							     else   
   							     {
   					        	    bundle.putLong(mDbHelper.KEY_ID, game_id);
   								    intent.putExtras(bundle); 
   			        		     }
							} catch (JSONException e) {
								e.printStackTrace();
							}
							break;
							
						case R.id.random :
							try {
							      game_id = sComm.gameChallenge(userName, user_id, OpponentName);
							      if (game_id > 0) {
							        	bundle.putLong(mDbHelper.KEY_ID, game_id);
										intent.putExtras(bundle); 
						    	  }
								  else {
										startError = true;
								  }
							} catch (JSONException e) {
								e.printStackTrace();
							}
							break;
							
						case R.id.device :
							try {
							    game_id = sComm.gameChallenge(userName, user_id, OpponentName);
							
								if (game_id > 0) {
									// pull down Device's tiles just as if 'he' accepted challenge
								    sComm.gameAcceptDevice(user_id, game_id, userName);
	
								    // pull down the remainder of the tiles and create local server 
								    sComm.gameLocalServer(game_id);
						        	bundle.putLong(mDbHelper.KEY_ID, game_id);
									intent.putExtras(bundle); 
					    		}
								else {
										startError = true;
								}
							} catch (JSONException e) {
								e.printStackTrace();
							}
							break;
							
						}
                    } catch (Throwable e) {
                        Log.e("ChUser", "Exception in server communication", e);
                    }
                    return startError;
				}

            }.execute();
	}
	
	
	@Override
	protected void onResume() {
		
		super.onResume();
		mDbHelper.open();
		
		populateData();
		getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
	}

	@Override
	protected void onPause() {
		
		super.onPause();
		mDbHelper.close();
	}
}
