package com.ulsanonline.triominoes;
/*
 * 02/22/2012 - v1.001 Initial release
 * 07/30/2012 - v3.3 - font size adjustments for large screens
 * 09/30/2012 - v3.4 - changed sound scheme, new logic for game ending 
 * 12/02/2012 - v3.7 - added settings button and help button to top
 * 12/13/2012 - v3.9 - make sure postInvalidate is called after thread finishes on  pass
 *                     also changed several XML files to accomodate small screen devices
 *                     game_id: int -> long
 * 01/06/2013 - v4.0 - enable switch from first fit to best fit for Android player
 * 01/30/2013 - v4.1 - ensure getSettings called on startup/init
 * 04/09/2013 - v4.2 - fix bugs
 * 09/30/2013 - v4.3 - cleanup chat/settings icons
 * 
 */ 
import java.util.ArrayList;
import org.json.JSONException;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.PointF;
import android.graphics.Rect;
import android.graphics.RectF;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.DisplayMetrics;
import android.util.FloatMath;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;
import android.content.DialogInterface;

import com.ulsanonline.triominoes.BoardClass;
import com.ulsanonline.triominoes.Settings;

public class DrawBoardView extends View {
	
   
    // map size in cells
    public static int mMapSize = 18;  // 16x16 triangles should be plenty
    
    // Offset to the upper left corner of the map
    private int mXOffset = 0;
    private int mYOffset = 0;
    
    // last touch point
    private int mXTouch = 0;
    private int mYTouch = 0;
    
 
    
    // screen play area
    // bottom 10% is for UI
    // lower 15% is for the shelf
    // upper 75% is game board
    private RectF uiZone;
    private RectF gameZone;
    private RectF shelfZone;
    private RectF lastPlayZone;
    private RectF tilesLeftZone;
    private RectF uiPlayBtn;
    private RectF uiPassBtn;
    private RectF uiRecallBtn;
    private RectF uiCheckBtn;
    private RectF uiChatBtn;
    private RectF uiSettingsBtn;
    private RectF uiHelpBtn;
    private static final float uiPct = 0.1f;
    private static final float gamePct = 0.75f;
    private static final float shelfPct = 0.15f;
	private static final int maxLocalPasses = 1;
     
    
    // subImage borders
    private int mStartRow;
    private int mMaxRow;
    private int mStartCol;
    private int mMaxCol;
    
    
    private long game_id;
    private int OppScore;
	private int userScore;
	private int user_id;
	private int opp_id;
	private String userName;
	private String OppName;
	private int turnScore;
	private int TilesPlayed = 0;
	private int currentTile;
	private int currentTilePlay;
	private int localPlayPasses;
	private Boolean madePlay;
	private DisplayMetrics screenMetrics;
    private Context mcontext;
    //private Point point;
    private int tileIdx;
    private int touchMode = NONE;
    private static final int MOVE = 2;
    private static final int ROTATE = 1;
    private static final int NONE = 0;
    private Boolean rotateLeft = false;
    static final int MIN_DISTANCE = 10;
    static final int DEG = 1;
    private float downX = 0, downY = 0;
    private float upX, upY;  // move start & end locations
    //private Paint p = new Paint();
    private String TAG = "DRAW";
    private static final int MaxRot = 5;  // hexagon pieces
    private  int BSIZE;  // size of an backgroundedge
    private  int TSIZE;  // size of a tile edge
    private int ALT ;    // altitude in pixels (sqrt(3)/2 * BSIZE)
    private ArrayList<BoardClass> board = new ArrayList<BoardClass>(); 
    private ArrayList<tileClass> tiles =  new ArrayList<tileClass>();   // all possible tiles
    private int ColRowGrid[][];           // this is a grid of col, row that points to the board indices 
    private int myHand[];
    private int myHandNumber;
    private int deviceHand[];
    private int deviceHandNumber;
    private int tilesPlayed[];
    private int tilesPlayedNumber;
    

	private int TileExtents[];
	private static final int LEFT = 0;
	private static final int TOP = 1; 
	private static final int RIGHT = 2;
	private static final int BOTTOM = 3; 

	private static final int usmTextSize = 10;
	private static final int vsmTextSize = 20;
    private static final int smTextSize = 25;
    private static final int mdTextSize = 35;
    private static final int lgTextSize = 45;
    private int controlTextSize;
    private int statusTextSize;
    private Boolean tilesRemain = true;
    private Boolean justBrowsing = false;
    private Bitmap bgTileDn;
    private Bitmap bgTileUp;
    private static Bitmap background;  // one big bitmap of many little triangles
    private Bitmap play_button;
    private Bitmap pass_button;
    private Bitmap recall_button;
    private Bitmap cancel_button;
    private Bitmap submit_button;
    private Bitmap check_button;
    private Bitmap chat_button;
    private Bitmap help_button;
    private Bitmap settings_button;
    private Bitmap chat_new_button;
    private int num_messages;
    private ServerCommunication sComm;

    private RectF rectM; // move
    private RectF rectL; // left corner
    private RectF rectR; // right corner
    private RectF rectA; // apex
    private PointF mid;
    private int BACKGROUND = 0xffd19c4f;       // a walnut color
    private int GAME_BACKGROUND = 0xff1898d8;  // a pleasing soft blue
    
    private ProgressDialog dialogUI; 
    private boolean challengeMade = false;     // used at end of game if rematch requested
    
    private triDbAdapter mDbHelper;

	 enum faces {left, right, above, below};

    
     enum tileId {t000, t001, t002, t003, t004, t005,
    	       //         6     7     8     9    10
                        t011, t012, t013, t014, t015,
                        //      11    12   13    14
                              t022, t023, t024, t025,
                              //      15   16    17
                                    t033, t034, t035,
                                    //     18    19
                                          t044, t045,
                                          //     20
                                                t055,
                     //   21    22    23   24    25
                        t111, t112, t113, t114, t115,
                        //      26    27   28    29
                              t122, t123, t124, t125,
                              //      30   31     32
                                    t133, t134, t135,
                                    //      33    34
                                          t144, t145,
                                          //      35
                                                t155,
                           //  36    37     38    39                     
                              t222, t223, t224, t225,
                              //     40     41    42
                                    t233, t234, t235,
                                    //      43    44
                                          t244, t245,
                                          //      45
                                                t255,
                                  //  46    47    48              
                                    t333, t334, t335,
                                    //      49    50
                                          t344, t345,
                                           //     51
                                                t355,
                                        //  52    53       
                                          t444, t445,
                                               // 54
                                                t455,
                                               // 55
                                                t555};

       
    
    
    /**
     * Construct object, initializing with any attributes we understand from a
     * layout file. These attributes are defined in
     * SDK/assets/res/any/classes.xml.
     * 
     * @see android.view.View#View(android.content.Context, android.util.AttributeSet)
     */

    
	public DrawBoardView(Context context, long in_game_id) {
		super(context);
		setFocusable(true); //necessary for getting the touch events
		game_id = in_game_id;
        mcontext = context;
        mDbHelper = new triDbAdapter(context.getApplicationContext());
        Log.d(TAG,"open on init");
		mDbHelper.open();
		
		InitBoardView(context);
        
	}
	
	
    // the method that is called when its time to render
    @Override protected void onDraw(Canvas canvas) {  	
    	canvas.drawColor(0xff1898d8);
    	Paint p = new Paint();
    	
    	if (Settings.ShowBoard) {    	

    		Rect src = new Rect(mStartCol,mStartRow,mStartCol + (int)gameZone.width(), mStartRow + (int) gameZone.height());    		
    		canvas.drawBitmap(background, src, gameZone, null);
    		
    		
    	}
    	
    	
        /*
    	if (Settings.ShowCenters) {   // shows grid locations
    		Log.d(TAG, "START: "+mStartCol+","+mStartRow);
	    	p.setTextSize(12);
	    	for (int i=0;i < board.size(); i++) {
	    		BoardClass h = board.get(i);
	    		p.setColor(Color.BLACK); 
	    		canvas.drawCircle(h.getPixelX()-mXOffset, h.getPixelY()-mYOffset, 5, p);
	    		p.setColor(Color.RED); 
	    		String loc = Integer.toString(h.getBoardCol()) + "," +
	    				     Integer.toString(h.getBoardRow());
	    		canvas.drawText(loc, h.getPixelX()-6-mXOffset, h.getPixelY()-6-mYOffset, p);
	    		loc = Integer.toString((int)h.getPixelX()) + " / " + Integer.toString((int)h.getPixelY());
	    		canvas.drawText(loc, h.getPixelX()-6-mXOffset, h.getPixelY()+6-mYOffset, p);
	    	}
    	}
    	*/
    	
    	if (madePlay) {
    		refreshUsersTiles();
    		refreshPlayedTiles();
    		refreshDeviceTiles();
    		if (myHandNumber ==0){ 
    			game_over();
    		}
    		if ((deviceHandNumber ==0)&& (mDbHelper.isPlayingDevice(game_id))) {
    			game_over();
    		}
    	}
    	     
    	// draw the tiles that are already played
    	int count = 0;
    	Cursor tc = mDbHelper.fetchPlayedTiles(game_id);
    	if (tc.getCount() > 0){
    		tilesPlayed = new int [tc.getCount()];
			tc.moveToFirst();
			do {
				int index = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_TILE));
				if (index >= 0) {
	     			tileClass tile = tiles.get(index);
					count++;
	     			canvas.drawBitmap(tile.getBitmap(), tile.getX()-(tile.getSizeX()/2)-mXOffset, tile.getY()-(tile.getSizeY()/2)-mYOffset, null);
					}
			} while (tc.moveToNext());
    	}
    	tilesPlayedNumber = count;
    	tc.close();
    	
        
    	//
    	// draw the User Interface Zone 
    	//
 	    p.setColor(BACKGROUND);
 	    canvas.drawRect(uiZone, p);  	
	    
        p.setColor(Color.BLACK);
        int ScoreX = (int) (0.6 * uiZone.width());
        
        int dia = 7;
        String statusline = "";
     	p.setTextSize(controlTextSize);
     	int textHeight = (int) p.getTextSize();
     	int ScoreY1 = textHeight;
     	OppScore = mDbHelper.getScore(game_id, opp_id);
        if (mDbHelper.isItMyTurn(game_id, user_id)) {
        	statusline = OppName;
 	        p.setColor(Color.WHITE);
 	     	canvas.drawText(">"+Integer.toString(userScore) + ":"+ userName, uiZone.left+ScoreX+dia,uiZone.top+ScoreY1, p);
 	     	p.setColor(Color.BLACK);
 	     	canvas.drawText(" "+Integer.toString(OppScore) + ":"+ OppName, uiZone.left+ScoreX+dia,uiZone.top+ScoreY1+(int)(textHeight*1.2), p);

 	     	canvas.drawBitmap(play_button, uiPlayBtn.left, uiPlayBtn.top, p);
 	     	canvas.drawText("Play", uiPlayBtn.left+(play_button.getWidth()/4),uiPlayBtn.bottom-(play_button.getHeight()/5), p); 
 	     	canvas.drawBitmap(pass_button, uiPassBtn.left, uiPassBtn.top, p);
 	     	canvas.drawText("Pass", uiPassBtn.left+(pass_button.getWidth()/5),uiPassBtn.bottom-(pass_button.getHeight()/5), p); 
 	    	canvas.drawBitmap(recall_button, uiRecallBtn.left, uiRecallBtn.top, p);
 	     	canvas.drawText("Recall", uiRecallBtn.left+(recall_button.getWidth()/5),uiRecallBtn.bottom-(recall_button.getHeight()/5), p); 
        } else {
        	statusline = userName;
        	p.setColor(Color.WHITE);
 	     	canvas.drawText(">"+Integer.toString(OppScore) + ":"+ OppName, uiZone.left+ScoreX+dia,uiZone.top+ScoreY1+(int)(textHeight*1.2), p);
        	p.setColor(Color.BLACK);
 	     	canvas.drawText(" "+Integer.toString(userScore) + ":"+ userName, uiZone.left+ScoreX+dia,uiZone.top+ScoreY1, p);
 	     	
 	     	canvas.drawBitmap(recall_button, uiRecallBtn.left, uiRecallBtn.top, p);
 	     	canvas.drawText("Recall", uiRecallBtn.left+(recall_button.getWidth()/5),uiRecallBtn.bottom-(recall_button.getHeight()/5), p); 
 	     	canvas.drawBitmap(check_button, uiCheckBtn.left, uiCheckBtn.top, p);
 	     	canvas.drawText("Check", uiCheckBtn.left+(check_button.getWidth()/5),uiCheckBtn.bottom-(check_button.getHeight()/5), p); 
        }
        

        p.setColor(Color.WHITE);
        p.setTextSize(statusTextSize);
        int width = TSIZE;
        int tilenm = mDbHelper.lastPlayed(game_id);
        int textWidth = (int)p.measureText(statusline);
        
        lastPlayZone = new RectF(gameZone.right-TSIZE,0, gameZone.right, TSIZE);
        if (TilesPlayed > 0) {
            p.setColor(Color.BLACK);
            //p.setColor(GAME_BACKGROUND);
            canvas.drawRect(lastPlayZone, p);

            
	        if (tilenm >= 0) {
	        	 // show last piece played
	        	 tileClass tile = tiles.get(tilenm);
	  		 	 canvas.drawBitmap(tile.getBitmap(), lastPlayZone.left, lastPlayZone.top, p);
	        	 
            }
	        p.setColor(Color.WHITE);
            if (textWidth > width) {
      		     canvas.drawText(statusline, lastPlayZone.left, lastPlayZone.centerY()+5,p);
	      	}
	      	else {
			     canvas.drawText(statusline, lastPlayZone.centerX()-p.measureText(statusline)/2,lastPlayZone.centerY()+5,p);
	      	}
            
	        if (tilenm < 0) {
	        	statusline = "passed";
	        	canvas.drawText(statusline, lastPlayZone.centerX()-p.measureText(statusline)/2, 
	        			                    lastPlayZone.centerY()+textHeight,p);
	        }
	        
        
        }  // else no status to report
        
        // draw the tilesLeft graphic
        p.setColor(Color.BLACK);
        //p.setColor(GAME_BACKGROUND);
        canvas.drawRect(tilesLeftZone, p);
        tileClass tile = tiles.get(0);       // 0 is the bare tile
   	    canvas.save();

   	    if (mDbHelper.getNumRemaining(game_id) > 0)
 		   canvas.drawBitmap(tile.getBitmap(), tilesLeftZone.left, tilesLeftZone.top, p);
	 	//canvas.restore();
	 	p.setTextSize(statusTextSize);
	 	p.setColor(Color.WHITE);
	 	String left =  " " +mDbHelper.getNumRemaining(game_id);
        canvas.drawText(left,tilesLeftZone.centerX()-(p.measureText(left)/2), 
        		             tilesLeftZone.centerY(),p);

	 	left = "tiles";
	 	canvas.drawText(left,tilesLeftZone.centerX()-(p.measureText(left)/2), 
	 			             tilesLeftZone.centerY()+p.getTextSize(),p);
        left = "left";
        canvas.drawText(left,tilesLeftZone.centerX()-(p.measureText(left)/2), 
	             tilesLeftZone.centerY()+(p.getTextSize()*2),p);
        
        // draw the shelf and the player's tiles 
    	p.setColor(Color.WHITE);
    	canvas.drawRect(shelfZone, p); 

        for (int index = 0; index < myHandNumber; index++ ) {
        	tile = tiles.get(myHand[index]);
        	int xoffset = 0;
        	int yoffset = 0;
        	if (tile.inPlay) {
        	 xoffset = mXOffset;
        	 yoffset = mYOffset;
        	}
            canvas.drawBitmap(tile.getBitmap(), (tile.getX()-xoffset)-tile.getSizeX()/2, (tile.getY()-yoffset)-tile.getSizeY()/2, null);
            
            if (Settings.ShowTouch) {
            	int dX;
            	int dY;

            	if (tile.inPlay) {
                	dX = tile.getX() - mXOffset;
                    dY = tile.getY() - mYOffset;
                }
                else {
                	dX = tile.getX();
                	dY = tile.getY();
                }
          		
        		int w = (int)( tile.getSizeX());
	    		int h = (int)( tile.getSizeY());
        		rectM = new RectF();
        		rectR = new RectF();
        		rectL = new RectF();
        		rectA = new RectF();
        		float w4 = w/4;
        		float h4 = h/4;
        		float w2 = w/2;
        		float h2 = h/2;
        		int exp = (tile.inPlay)?5:0;
        		if (tile.isFaceDown())   {
 	     		   rectM.set(dX-w4,    dY-h2,    dX+w4,    dY+h4);   // this tile faces up, reverse bounds
 	     		   rectL.set(dX-w2-w4-exp, dY-h2-h4-exp, dX-w2+w4, dY-h2+h4+exp);  // rotate touch upper left
 	     		   rectR.set(dX+w2-w4, dY-h2-h4-exp, dX+w2+w4+exp, dY-h2+h4+exp ); // rotate touch Ur
 	     		   rectA.set(dX-w4-exp,    dY+h2-h4, dX+w4+exp,    dY+h2+h4+exp ); // rotate touch Nadir
 	     		}
 	     		else {                         
 	     		   rectM.set(dX-w4,    dY-h4,    dX+w4,    dY+h2);     // check the base for move	
 	     		   rectL.set(dX-w2-w4-exp, dY+h2-h4-exp, dX-w2+w4, dY+h2+h4+exp ); // rotate touch LL
 		     	   rectR.set(dX+w2-w4, dY+h2-h4-exp, dX+w2+w4+exp, dY+h2+h4+exp ); // touch LR
 		     	   rectA.set(dX-w4-exp,    dY-h2-h4-exp, dX+w4+exp,    dY-h2+h4);  // rotate touch Apex		     		
 	     		}
         		
	     		
     			p.setColor(Color.BLUE);// blue move points are around the bulk
     			p.setAlpha(32);
	     		canvas.drawRect(rectM, p);
	     		
	     		p.setColor(Color.RED);//  rot points are around the corners
	     		p.setAlpha(32);
	     		canvas.drawRect(rectL,p);
	     		canvas.drawRect(rectR,p);
	     		canvas.drawRect(rectA,p);
            }
            
        }
        
        // draw Android's tiles 
        if (Settings.ShowAndroidTiles) {
        	if (deviceHandNumber > 0) {
	        	canvas.save();
	        	canvas.scale(0.5f, 0.5f,shelfZone.left,shelfZone.top);
	            int lastX = 0;
	            int lastY = 0;
	        	for (int i=0; i<deviceHandNumber; i++) {
	        		tile = tiles.get(deviceHand[i]);
	        		canvas.drawBitmap(tile.getBitmap(), (tile.getX()-tile.getSizeX()/2), (tile.getY()-tile.getSizeY()/2), null);
	        		lastX = (tile.getX()+tile.getSizeX()/2);
	        		lastY = tile.getY();
	        	}
	        	
	            	
	            p.setColor(Color.BLACK);
	            p.setTextSize(statusTextSize*2);
	        	String label = "Android's Tiles";
	        	canvas.drawText(label,lastX,lastY,p);
	        	canvas.restore();
        	}
        	
        }
        
        p.setTextSize(statusTextSize);
        p.setAlpha(255);
        // draw the settings and help buttons
        canvas.drawBitmap(settings_button,uiSettingsBtn.left, uiSettingsBtn.top,p);
        canvas.drawBitmap(help_button, uiHelpBtn.left, uiHelpBtn.top,p);
        
        // draw the uiChatZone 
        //canvas.drawRect(uiChatBtn, p);
        canvas.drawBitmap(chat_button, uiChatBtn.left, uiChatBtn.top, p);
        if (num_messages > 0){
        	canvas.drawBitmap(chat_new_button, uiChatBtn.left, uiChatBtn.top, p);
            p.setColor(Color.WHITE);
        	canvas.drawText(Integer.toString(num_messages), uiChatBtn.left+10, uiChatBtn.top+10, p);
        }
       // p.setColor(Color.BLACK);
       // String chat = "chat";
       // canvas.drawText(chat, uiChatBtn.left + uiChatBtn.width()/2 - (p.measureText(chat)/2) , uiChatBtn.centerY(), p);
        
        
        if (mDbHelper.gameOver(game_id)) {
			askRematch();
			// need some logic here. It's a solo game and would like to store 
    		// scores from user and Android 
			
		}
        
        if (madePlay) {
    		madePlay = false;
    		invalidate();
    	}
     	
    }



	// events when touching the screen
    @Override
	public boolean onTouchEvent(MotionEvent event) {
        int eventaction = event.getAction(); 
        
        
        int X = (int)(event.getX()); 
        int Y = (int)(event.getY()); 

        //Log.d(TAG,"x= " +X + "    Y= "+Y);

        
         
         
        switch (eventaction ) { 
        


        case MotionEvent.ACTION_DOWN: // touch down so check if the finger is on a tile
        	tileIdx = -1;
        	downX = event.getX();
            downY = event.getY();
            
/*
 * why do i have two here?
 */
            // store the current touch coordinates for scroll calculation
            mXTouch = (int) event.getX();
            mYTouch = (int) event.getY();
            
            // check first for any buttons pressed and perform required actions
        	//
            if (uiChatBtn.contains(downX, downY)) { 
            	Log.d(TAG,"Chat Button pressed");
            	SoundManager.playSound(SoundManager.Sounds.click.sound(), 1);

            	Bundle bundle = new Bundle();
	        	bundle.putLong(mDbHelper.KEY_ID, game_id);
				bundle.putInt(mDbHelper.KEY_USERID, user_id);
     
				Intent i = new Intent(mcontext,InGameChat.class);
				i.putExtras(bundle); 
				mcontext.startActivity(i);
            }
            
            if (uiSettingsBtn.contains(downX,downY)) {
            	Intent i = new Intent(mcontext,ChangeSettings.class);
            	mcontext.startActivity(i);
            }
            
            if (uiHelpBtn.contains(downX,downY)) {
            	Intent i = new Intent(mcontext,Help.class);
            	mcontext.startActivity(i);
            }
            
            
            if (mDbHelper.isItMyTurn(game_id, user_id)) {
	            if (uiPlayBtn.contains(downX, downY)) {                // play & pass / check are mutually exclusive
	            	Log.d(TAG,"Play button touched ");
	            	
	            	// first check if it's this player's turn
	            	if ( (mDbHelper.isItMyTurn(game_id, user_id)) || (TilesPlayed == 0)) {
		            	if (currentTile >= 0) {
		            		
		            		if (!justBrowsing) {
		            			playTile(currentTile, user_id);
		            		    
			            		
			            		localPlayPasses = 0;
			            		currentTile = -1;
			            		
			            		// player had his turn and played a tile.  If playing the device, now it's device's turn
			            		if (!mDbHelper.gameOver(game_id)){
			            			playDeviceTile();
			            		}
			            		madePlay = true;
		            		}
		            	}
		            	else if (currentTilePlay >= 0) {
			 	    		Toast.makeText(getContext(), "You cannot play with that tile placement.", Toast.LENGTH_SHORT).show();
		                	SoundManager.playSound(SoundManager.Sounds.noplay2.sound(), 1);
	
		            	}
	            	}
	            	else   
	            	{
		 	    		Toast.makeText(getContext(), "You have already played. Please wait for your opponent", Toast.LENGTH_SHORT).show();
	                	SoundManager.playSound(SoundManager.Sounds.noplay2.sound(), 1);
	            	}
	            }
	            if (uiPassBtn.contains(downX, downY)) {
	            	Log.d(TAG,"Pass button touched ");
	            	SoundManager.playSound(SoundManager.Sounds.click.sound(), 1);
	            	if (mDbHelper.isItMyTurn(game_id, user_id)) {
	            		if (!mDbHelper.tilesRemaining(game_id)) {
	    	 	    		Toast.makeText(mcontext, "No more tiles.", Toast.LENGTH_SHORT).show();
	    	 	    		if (mDbHelper.lastPlayed(game_id) < 0) { // opponent passed, me too
	                            game_over();
	    	 	    		}
	    	 	    		/*
	    	 	    		 * 
	    	 	    		 */
	                        if ((localPlayPasses >= maxLocalPasses) || // if we've passed enough, game over
	                            (mDbHelper.isSoloGame(game_id)) ) {    // or if solo game 
	                        	game_over();
	                        	
	                        }
	            		}
	            		// tell opponent and pick up new tile
	                    if ((!justBrowsing) && (!mDbHelper.gameOver(game_id))) {
	                    	passPlay(user_id);   
	                    	// player had his turn and passed.  If playing the device, now it's device's turn
		            		playDeviceTile();
		            		
		            		madePlay = true;
	                    }
	            	}
	            	else {
		 	    		Toast.makeText(getContext(), "You have already played. Please wait for your opponent", Toast.LENGTH_SHORT).show();
	                	SoundManager.playSound(SoundManager.Sounds.noplay2.sound(), 1);

	            	}
	            }
            }
            else {                         // not my turn. But you can check to see if the other guy played 
	            if (uiCheckBtn.contains(downX, downY)) {
	 	    		dialogUI.show();
	 	   		    Thread checkUpdate = new Thread() {  
	 	   		        public void run() {
		 	   		    	//
		 	   		    	// YOUR LONG CALCULATION (OR OTHER) GOES HERE
		 	   		    	//
		 	   		    	Looper.prepare();
							try {
								if (sComm.checkplay(game_id, 0)) {
									madePlay = true;
									postInvalidate();
									OppScore = mDbHelper.getScore(game_id, opp_id);
                                    
									
									
								}
							} catch (JSONException e) {
								e.printStackTrace();
							}
							
							//Looper.loop();
					    	handler.sendEmptyMessage(0);
					    }
					};
					checkUpdate.start();
					
					
				
            	}
            }
            
            if (uiRecallBtn.contains(downX, downY)) {
            	SoundManager.playSound(SoundManager.Sounds.click.sound(), 1);
            	Log.d(TAG,"Recall button touched ");
            	if (currentTile >= 0) {
            		removeTileFromBoard(currentTile); 
                	SoundManager.playSound(SoundManager.Sounds.recall.sound(), 1);

            	}
            	if (currentTilePlay >= 0) {
            		removeTileFromBoard(currentTilePlay ); 
                	SoundManager.playSound(SoundManager.Sounds.recall.sound(), 1);

            	}
            	refreshUsersTiles();
            }
            
            for (int index = myHandNumber-1; index >=0 ; index-- ) {
        		
            	tileClass tile = tiles.get(myHand[index]);
            	// only need to consider the tiles that are in a player's shelf 
            	int dX;
            	int dY;
            	
            	//
            	// can this be changed to a terniary on
            	// the inplay set/get function?  this is ?
            	
            	//
            	if (tile.inPlay) {
                	dX = tile.getX() - mXOffset;
                    dY = tile.getY() - mYOffset;
                }
                else {
                	dX = tile.getX();
                	dY = tile.getY();
                }
          		
        		int w = (int)( tile.getSizeX());
	    		int h = (int)( tile.getSizeY());
        		rectM = new RectF();
        		rectR = new RectF();
        		rectL = new RectF();
        		rectA = new RectF();
        		int exp = (tile.inPlay)?7:0;
        		float w4 = w/4;
        		float h4 = h/4;
        		float w2 = w/2;
        		float h2 = h/2;
	     		if (tile.isFaceDown())   {
	     		   rectM.set(dX-w4,    dY-h2,    dX+w4,    dY+h4);   // this tile faces up, reverse bounds
	     		   rectL.set(dX-w2-w4-exp, dY-h2-h4-exp, dX-w2+w4, dY-h2+h4+exp);  // rotate touch upper left
	     		   rectR.set(dX+w2-w4, dY-h2-h4-exp, dX+w2+w4+exp, dY-h2+h4+exp ); // rotate touch Ur
	     		   rectA.set(dX-w4-exp,    dY+h2-h4, dX+w4+exp,    dY+h2+h4+exp ); // rotate touch Nadir
	     		}
	     		else {                         
	     		   rectM.set(dX-w4,    dY-h4,    dX+w4,    dY+h2);     // check the base for move	
	     		   rectL.set(dX-w2-w4-exp, dY+h2-h4-exp, dX-w2+w4, dY+h2+h4+exp ); // rotate touch LL
		     	   rectR.set(dX+w2-w4, dY+h2-h4-exp, dX+w2+w4+exp, dY+h2+h4+exp ); // touch LR
		     	   rectA.set(dX-w4-exp,    dY-h2-h4-exp, dX+w4+exp,    dY-h2+h4);  // rotate touch Apex		     		
	     		}
        		
        		if(rectM.contains(X, Y)) {    // You hit the move area of the box, allow dragging...
        			tileIdx = index;
        			touchMode = MOVE;
         		  	 Log.d(TAG,"tile "+tileIdx+ " touched to move");  	 	    	                      
                   //// break;
        		}
        		if ( (rectR.contains(X, Y)) || (rectL.contains(X, Y)) || (rectA.contains(X, Y)) )
        		{
        		                 // You hit the rotate area of the box, allow rotating...
        			tileIdx = index;
        			touchMode = ROTATE;
        			rotateLeft = (rectL.contains(X, Y));
        			
       		  	 Log.d(TAG,"tile "+tileIdx+ " touched to rotate");

                //    break;
            		
        		}
            	

              }
            
            if (shelfZone.contains(X,Y)) {
            	Log.d(TAG,"shelfzone touched");
            }
            
            if (gameZone.contains(X,Y)) {
            	Log.d(TAG,"gamezone touched ");
            }
              
       
             break; 
             

     	 
       case MotionEvent.ACTION_MOVE:  

    	   if (touchMode == MOVE) {
	         	// move the tile the same as the finger
    		   int xoffset = 0;
    		   int yoffset = 0;
		       if (tileIdx >= 0) {
		       	tileClass tile = tiles.get(myHand[tileIdx]);
		       	// set limits of physical screen
		       	if (tile.inPlay) {
		       		xoffset = mXOffset;
		       		yoffset = mYOffset;
		       	}
		       	if (X < 0)
		       		tile.setX(0);
		       	else if (X > (gameZone.width() ))
		       		tile.setX((int) gameZone.width() );
		       	else tile.setX(X+xoffset);
		       	
		       	if (Y < 0)
		       		tile.setY(0);
		       	else if (Y > (shelfZone.bottom +  tile.getSizeY() ))
		       		tile.setY((int) (shelfZone.bottom +  tile.getSizeY()));
		       	else tile.setY(Y+yoffset);
		       	
		       }
    	   } 
    	   if (touchMode == ROTATE) {
    		   Log.d(TAG,"action move = rotate");
    	   }
    	   
    	   /*
    	    * 
    	    * wasn't there supposed to be something here for ROTATE?
    	    * 
    	    */
    	   
    	   if ((touchMode != MOVE) && (touchMode != ROTATE)) {
               // it wasn't a move event for a tile. assume it was a scroll event on the game board
               
              // mIsMoving = true;
               
               // get the new offset
               mXOffset += mXTouch - (int) event.getX();
               mYOffset += mYTouch - (int) event.getY();
               
               // secure, that the offset is never out of view bounds
               if (mXOffset < 0) {
                   mXOffset = 0;
               } //else if (mXOffset > mMapSize * BSIZE - getWidth()) {
                 //  mXOffset = mMapSize * BSIZE - getWidth();
				else if (mXOffset > (background.getWidth() - ((int)gameZone.width()) ) ){
                     mXOffset = background.getWidth()-((int)gameZone.width());
               }
               if (mYOffset < 0) {
                   mYOffset = 0;
               }// else if (mYOffset > mMapSize * BSIZE - getHeight()) {
                //   mYOffset = mMapSize * BSIZE - getHeight();
               else if (mYOffset > background.getHeight() - gameZone.height()) {
            	   mYOffset = background.getHeight() - (int)gameZone.height();
               }
               
               // store the last position
               mXTouch = (int) event.getX();
               mYTouch = (int) event.getY();
               
               calculateLoopBorders();
               
             } 
	   	
	       break; 

	   case MotionEvent.ACTION_UP: 
	  		// touch drop - motion is finished for this tile save final X,Y, or set rotate from swipe
		    upX = event.getX();
            upY = event.getY();
            
	 	    if (touchMode == ROTATE) {
                // set the rotation on the tile
	 	    	if (tileIdx >= 0) {
	 	    		turnScore=-1;
	 	    		if (rotateLeft) 
	 	    		    rotateTile(myHand[tileIdx], -DEG);
	 	    		else
	 	    			rotateTile(myHand[tileIdx], DEG);
	 	    		
	 	    		if (tiles.get(myHand[tileIdx]).inPlay)  {
	 	    			
                        // check that rotation is still ok for this spot on the board
	 	    			if (tiles.get(myHand[tileIdx]).isFaceDown() == board.get(tiles.get(myHand[tileIdx]).board_index).isfaceDown()) {
	 	    		       turnScore = placeTileOnBoard(tiles.get(myHand[tileIdx]).board_index, myHand[tileIdx]);
	 	    			}
	 	    		
		 	    		if ( turnScore >= 0) {
		            		currentTile = myHand[tileIdx];  // this is a good tile placement, save this 
		            		if (!Settings.EnableSound) {
		            			if (Settings.ShowScorePopUps) {
 		 	    		           Toast.makeText(getContext(), "OK", Toast.LENGTH_SHORT).show();
		            			}
		            		}
		 	    		    SoundManager.playSound(SoundManager.Sounds.ok.sound(), 1);
		 	    		}
		 	    		else  {
		 	    			if (!Settings.EnableSound) {
		            		   Toast.makeText(getContext(), "You cannot place it in that spot", Toast.LENGTH_SHORT).show();
		 	    			}
		            		SoundManager.playRandomBadMoveSound();
		            		currentTilePlay = myHand[tileIdx];
		            		currentTile = -1;              //  clear it

		            	}
		 	    			
	 	    		}
	 	    	}
	 	    	touchMode = NONE;
            	
    	    }
	 	    else if (touchMode == MOVE) {
	 	    	// we've been touching a tile, attempting to move it. 
	 	    	// movement is done, so let's first make sure it's on the board
	 	    	// and if not in the UI or Shelf Zones we'll then
	 	    	// stash final location of the tile onto a grid location.
	 	    	// find the closest board grid to drop this tile
	 	    	
	 	    	if (gameZone.contains(X, Y) ){
		 	    	
	 	    		if (tileIdx >= 0) {
			 	    	int bestmatch = findClosestOpenSpot(myHand[tileIdx], X, Y);
			 	    	
			 	    	if ((currentTile >= 0) && (currentTile!=myHand[tileIdx])) {
			 	    		//player already has a tile in play. Wrist-spanking mode enabled
			 	    		if (!mDbHelper.isPlayingDevice(game_id)) {
			 	    		   Toast.makeText(getContext(), "You have already placed one tile. You must recall it to place this one", Toast.LENGTH_SHORT).show();
			 	    		}
			 	    	} 
			 	    	
			            	//
				 	    	// "SNAP" this tile into place. Cue audio snick
				 	    	//
		            	SoundManager.playSound(SoundManager.Sounds.droptile.sound(), 1);
		            	turnScore = placeTileOnBoard(bestmatch, myHand[tileIdx]);
		 	    	
		 	    	
		            	if ( turnScore >= 0) {
		            		currentTile = myHand[tileIdx];  // this is a good tile placement, save it
		            		if (!Settings.EnableSound) {
		            			if (Settings.ShowScorePopUps) {
	 	            		       Toast.makeText(getContext(), "OK", Toast.LENGTH_SHORT).show();
		            			}
		            		}
		            		SoundManager.playSound(SoundManager.Sounds.ok.sound(), 1);
		            	}
		 	    
		            	else {
		            		if (!Settings.EnableSound) {
		            		   Toast.makeText(getContext(), "You cannot place it in that spot", Toast.LENGTH_SHORT).show();
		            		}
		            		SoundManager.playRandomBadMoveSound();
		            		currentTilePlay = myHand[tileIdx];  // still have a tile in play, just not a valid one
		            		currentTile = -1;              // clear it
	
		            	}
	 	    		}
	 	    	} // touch was in UI or Shelf
	 	    	if (shelfZone.contains(upX, upY)) {
	 	    		// if we dropped a tile we were moving onto the shelf, 
	 	    		// then we need to take it out of play
	 	    		if (tileIdx >= 0) {
	 	    		  tileClass tile = tiles.get(myHand[tileIdx]);
	 	    		  tile.setX((int)upX);
	 	    	      tile.setY((int)upY);
	 	        	  tile.played = false;
	 	        	  tile.inPlay = false;
	 	        	  tile.board_index = -1;
	 	    		}
	 	    	}
	          
	 	    	touchMode = NONE;
	 	    }
	 	    
	 	    	
	        break; 
   } 
   // redraw the canvas
   invalidate(); 
   return true; 

}



    
    
    /*
     * given an index into the tiles, and a X/Y screen location, 
     * find an open spot. Returns a pointer to the board 
     */
    private int findClosestOpenSpot(int tileIdx, int X, int Y) {
    	
    	tileClass myTile = tiles.get(tileIdx);
    	String grid = "";
    	int bestmatch = -1;
    	float closest = 10000;
    	int xoffset = mXOffset;
    	int yoffset = mYOffset;


    	//int myRow = (int)((myTile.getY()+yoffset) / BSIZE);
    	int myRow = ((Y+yoffset) / BSIZE);
    	for (int col = 0; col < mMaxCol; col++) {
	    	//for (int index = 0; index < board.size(); index++ ) {	
	    	    // consider only board grid locations that are :
 	    	//     -- not occupied already by a tile
 	    	//     -- are in the same row as the tile I just touched/dragged
    		int index = ColRowGrid[col][myRow];
    		BoardClass h = board.get(index);
 	    	if (!h.occupied) {
 	    		
	                // find closest grid location

 	    		float deltaX = (h.getPixelX() - (X + xoffset));
 	    		float deltaY = (h.getPixelY() - (Y + yoffset));
          	    float dist = (float) Math.sqrt(deltaX * deltaX  + deltaY * deltaY );
          	    
   		       // Log.d(TAG, bestmatch + " DISTANCE--------> " + dist + "   X:" + deltaX + "--" + X +
   			   //      "   Y:" + deltaY + "--" + Y + " at " + col +  "/" + myRow);	
   		        
          	    if (dist < closest) {
          		    closest = dist;
          		    bestmatch = index;
          		    
          	    }
 	    	}
    	}	
    	if (bestmatch >= 0) {
	    	Boolean facedn = board.get(bestmatch).isfaceDown();
	    	if (facedn !=  myTile.isFaceDown()) {	
	    		int currentRotation = myTile.getRotate();
	    		currentRotation = ((currentRotation+3) > MaxRot)?0:(currentRotation+3);
	    		myTile.setRotate(currentRotation, true);  // re-orient this tile so that it's facing up or down correctly
	    	}
	    	Log.d(TAG, "Placing tile into " + grid + " Grid: " +
	    			bestmatch +" at "+ board.get(bestmatch).getBoardCol() + 
	    			" " + board.get(bestmatch).getBoardRow());
    	}
    	return bestmatch;
    }
    
    /*
     * rotate the given tile by the integer rotation
     */
    private void rotateTile(int index, int rotation) {

     	tileClass tile = tiles.get(index);
 		int currentRotation = tile.getRotate();
        currentRotation += rotation;
        if (currentRotation > MaxRot) currentRotation = 0; // roll-over to only possible values
        if (currentRotation < 0) currentRotation = MaxRot;
        
     	
     	// "FLICK" this tile into proper rotation. Cue audio snick
     	tile.setRotate(currentRotation,true);
     	String orient = (tile.isFaceDown()) ? " down" : " up";
     	Log.i(TAG, "rotated to position " + tile.getRotate() + orient);
     	SoundManager.playSound(SoundManager.Sounds.rotate.sound(), 1);

    }
    
    /*
     * make sure scrolling the game board doesn't go too far
     */
    private void calculateLoopBorders() {
        mStartRow = mYOffset / 1;
        //mMaxRow = (int) Math.min(mMapSize, mStartRow + Math.floor(getHeight() / BSIZE) + 2);
        mStartCol = mXOffset / 1;
        //mMaxCol = (int) Math.min(mMapSize, mStartCol + Math.floor(getWidth() / BSIZE) + 2);

    }
    
    /*
     * passes play to the opposing user. If another tile is available, it is
     * pulled from the server and placed in the local user's hand and the
     * score is updated.
     */
    private void passPlay(int who) {
    	final int which = who;
		SoundManager.playSound(SoundManager.Sounds.pass.sound(),1);

    	if ((mDbHelper.isSoloGame(game_id)) || (mDbHelper.isPlayingDevice(game_id))) {
    		if (mDbHelper.drawTile(game_id, who)) {
    			if (user_id == which) {
			         userScore = mDbHelper.getScore(game_id, user_id);
			    }
			    else {
			         OppScore = mDbHelper.getScore(game_id, which);
			   	}
			}
			else { 
                 tilesRemain = false;
			}
    	}
    	else {
	    	dialogUI.show();
			Thread checkUpdate = new Thread() {  
			    public void run() {
			    	//
			    	// YOUR LONG CALCULATION (OR OTHER) GOES HERE
			    	//
			    	Looper.prepare();
			    	try {
						if (sComm.passplay(game_id, which)) {					    	
					    	userScore = mDbHelper.getScore(game_id, user_id);
					    	postInvalidate();
					    	madePlay = true;
						}
						else { 
	                        tilesRemain = false;
						}
					} catch (JSONException e) {
						Toast.makeText(mcontext, "Something went wrong getting a new tile", Toast.LENGTH_LONG).show();
						e.printStackTrace();
					}
					//Looper.loop();
			    	handler.sendEmptyMessage(0);
				    }
				};
			checkUpdate.start();
    	}
		madePlay = true;
		if (!tilesRemain) {
			// return false says no tiles for you, kiddo
			Toast.makeText(mcontext, "There are no tiles left", Toast.LENGTH_SHORT).show();
		}
		invalidate();
    }


    /*
     * finalizes the playing of a tile, places it permanently on the board,
     * removes it from the player's hand and updates the score.
     */
    private void playTile(int pTile, int who) {
        final int ID, row, col, rot, complete;
        final String name;
        final Boolean facedn;
        final int which=who;
        final int whose;

        SoundManager.playSound(SoundManager.Sounds.point2.sound(), 1);
		
    	 
		TilesPlayed++;
		
		// not sure how this happens
		if ((pTile < 0) || (pTile >= tiles.size())) {
			Toast.makeText(mcontext, "Something went wrong placing a tile", Toast.LENGTH_SHORT).show();
			return;
		}
		
		
		// stuff the dB with the pertinent info 
		tileClass tile = tiles.get(pTile);
		if ((tile.board_index < 0) || (tile.board_index > board.size())) {
			Toast.makeText(mcontext, "Something went wrong placing a tile", Toast.LENGTH_SHORT).show();
			return;
		}
		BoardClass spot = board.get(tile.board_index);
		spot.occupied = true;

	    tile.setX(spot.getPixelX());
	    tile.setY(spot.getPixelY());
        // set this tile to be "played" and set it's location is on the board
        tile.played = true; 
		
        if (user_id != who) {
        	mDbHelper.DeleteDeviceTile(game_id, pTile);
        	complete = (deviceHandNumber == 0)?1:0;
        	if (deviceHandNumber == 0) {
    			// last tile was just played.  Add 25 to score
    			turnScore += 25;
    		} 
        	OppScore += turnScore;
        	whose = OppScore;
        }
        else {
  		    mDbHelper.DeleteTile(game_id, pTile);
		    complete = (myHandNumber == 0)?1:0;
		    if (myHandNumber == 0) {
				// last tile was just played.  Add 25 to score
				turnScore += 25;
			} 
		    userScore += turnScore;
		    whose = userScore;
        }
        madePlay = true;
		
		
		int tilesRemain = (mDbHelper.tilesRemaining(game_id))?1:0;
		// update local dB
		mDbHelper.UpdateLastPlayed(game_id, user_id, which, whose, tile.getID(),
		         tile.fname,spot.getBoardRow(),spot.getBoardCol(),tile.rotate,tile.isFaceDown(), turnScore, tilesRemain, complete);

		// communicate the same info with the central server
		ID = tile.getID();
		name = tile.fname;
		row =  spot.getBoardRow();
		col = spot.getBoardCol();
		rot = tile.getRotate();
		facedn = tile.isFaceDown();
		
		if ((!mDbHelper.isSoloGame(game_id)) && (!mDbHelper.isPlayingDevice(game_id))) {  // only communicate with server if non-local game
			dialogUI.show();
			Thread checkUpdate = new Thread() {  
			    public void run() {

			    	Looper.prepare();
			    	try {
						sComm.playTile(game_id, user_id, which, whose, ID ,name, row, col, rot, facedn, complete);
					} catch (JSONException e) {
						Toast.makeText(mcontext, "Something went wrong in communicating. Your data was saved locally", Toast.LENGTH_LONG).show();
						e.printStackTrace();
					}
					
					//Looper.loop();
			    	handler.sendEmptyMessage(0);
			    }
		    };
		    checkUpdate.start();
		}

		if (turnScore - (spot.vertices[0] + spot.vertices[1] + spot.vertices[2]) == 50) { 
    	     SoundManager.playSound(SoundManager.Sounds.woohoo.sound(), 1);
		}else if (turnScore - (spot.vertices[0] + spot.vertices[1] + spot.vertices[2]) == 40) { 
			 SoundManager.playSound(SoundManager.Sounds.wow.sound(), 1);
		}

		
		if (user_id == who) {
			if (Settings.ShowScorePopUps) {
	           Toast.makeText(mcontext, "You scored "+turnScore+" points!", Toast.LENGTH_SHORT).show();
			}
	        if (myHandNumber == 0) {
	        	game_over();
	        }
		}
		else {
			if (Settings.ShowScorePopUps) {
			    Toast.makeText(mcontext, "ANDROID scored "+turnScore+" points!", Toast.LENGTH_SHORT).show();
			}
			if (deviceHandNumber == 0) {
				game_over();
			}
		}
		turnScore = 0;

		invalidate();
		
    }
    
    
    /*
     *  if the user is playing against the machine, here is where the machine
     *  gets its turn
     */ 
    
    private static class placedTile {
    	public static int row = -1;
    	public static int col ;
    	public static int rot;
    	public static int index;
    	public static int score;
    }
    private void playDeviceTile() {
    	
    	if (mDbHelper.isPlayingDevice(game_id)) {
    		
    		// make sure I'm seeing all played tiles
    		refreshPlayedTiles();
  
    		LongRunTask searchTask = new LongRunTask();

    		searchTask.execute();



    	}
    }
    
    /***************************************
     * Async Task
     * @return
     */

    
    protected class LongRunTask extends AsyncTask<Void, Integer, placedTile> {
    	
    	final ProgressDialog dialog = new ProgressDialog(mcontext);
    	
    	@Override
    	protected void onPreExecute() {
    		
    		dialog.setMessage("Android is thinking..."); 
    		dialog.show();
    	}

		@Override
		protected placedTile doInBackground(Void... params) {
			placedTile result = new placedTile();
			
	    	Boolean found = false;
	    	tileClass tile;
	    	int tileNum;
	    	BoardClass spot;
	    	int rot;
	    	int maxrot;
	    	int score = -1;
	    	int i = 0;
	    	int bestRot = -1;
	    	int bestCol = -1;
	    	int bestRow = -1;
	    	int bestTile = -1;
	    	int bestScore = -1;
	    	Boolean noSetBitMap = false;
	    	Boolean setBitMap = true;
	    	
	    	// Loop through all open spots on the board, and check each tile in each rotation
	    	// user has a choice of 
	    	//   - easy play - Android uses first fit placement
	    	//   - hard play - Android uses best fit placement
	    	// Loop returns when a valid (first/best) placement and score occurs or no play possible 
	    	
	    	while ((i < deviceHandNumber) && (!found)) {
	    		tileNum = deviceHand[i];
	    		tile = tiles.get(tileNum);
	    		int row = TileExtents[TOP];
	    		int col = TileExtents[LEFT];
	    		while ((row <= TileExtents[BOTTOM]) && (!found)) {
	    			while ((col <= TileExtents[RIGHT]) && (!found)) {
	    				
	    				spot = board.get(ColRowGrid[col][row]);
	    				if (!spot.occupied) {
	        				if (spot.facedown) {
	        					rot = 3;
	        					maxrot = 6;
	        				}
	        				else {
	        					rot = 0;
	        					maxrot = 3;
	        				}
	        				while ((rot < maxrot) && (!found)) {
	        					// set the rotation and check for a score
	        					tile.setRotate(rot, noSetBitMap); // when searching, don't bother setting bitmap
	        					//Log.d(TAG,"CRT "+col+" "+row+" "+rot+" "+tile.getScore(0)+" "+tile.getScore(1)+" "+tile.getScore(2));
	        					
	        					score = placeTileOnBoard(spot.index, tileNum);
	        					
	        					if (score >= 0) {
	        						
	        						if (Settings.AndroidPlayDifficult == false) {
		        						// hey! we found a spot. Play it right away.
		        						//tile.setRotate(rot, setBitMap); // set the bitmap so scores display correctly
		        						turnScore = score;
		        						currentTile = tileNum;
		        						bestRot = rot;
	    						    	bestCol = col;
	    						    	bestRow = row;
	    						    	bestTile = tileNum;
	    						    	bestScore = score;
		        						found = true;                    // set found so we exit loop
		            					Log.d(TAG,"CRT "+col+" "+row+" "+rot+" "+tile.getScore(0)+" "+tile.getScore(1)+" "+tile.getScore(2));

	        						}
	        						else {
	        							// we found a spot, if it's the best so far, save it for later 
	        							if (score > bestScore) {
	        								bestRot = rot;
	        						    	bestCol = col;
	        						    	bestRow = row;
	        						    	bestTile = tileNum;
	        						    	bestScore = score;
	        	        					Log.d(TAG,"CRT "+col+" "+row+" "+rot+" "+tile.getScore(0)+" "+tile.getScore(1)+" "+tile.getScore(2));

	        							}
	        						}
	        					}
	        					rot++;
	        				}
	        				
	    				}
	    				col++;

	    			}
	    			row++;
	    			col=TileExtents[LEFT];
	    		}
	    		i++;

	        }
	    	result.col = bestCol;
	    	result.row = bestRow;
	    	result.rot = bestRot;
	    	result.score = bestScore;
	    	result.index = bestTile;
	    	if (Settings.AndroidPlayDifficult == false)  {
	    		return result; // everything is already set to go if there was a spot found or not.
	    	}
	    	else if (bestScore >= 0) { 
	    		
	    		return result;
	    	}
	    	else
	    		return result;
	    	
	   
		}
		
		@Override
        protected void onPostExecute(placedTile deviceTileInfo) {
			
			if (placedTile.index == -1) {
            	// never found a tile to play. We have to pass
            	passPlay(1); // user_id '1' is always ANDROID
            }
        	else {
        		tileClass tile = tiles.get(placedTile.index);
        		tile.inPlay = true;
        		tile.setRotate(placedTile.rot, true);
        		;
        		turnScore = placeTileOnBoard(board.get(ColRowGrid[placedTile.col][placedTile.row]).index, placedTile.index);
        		currentTile = placedTile.index;
				playTile(currentTile,1);   // tileNum is the index of the tile we found a spot for. '1' is the ANDROID player

        	}
        	dialog.dismiss();
        	postInvalidate();
        	invalidate();
			
			
		}

    }
    
    /***************************************
     * Async Task
     * 
     */
    

    
    /*
     * whether by user recall or by inappropriate placement, this tile comes off and goes back
     * to player's Shelf
     */
    private void removeTileFromBoard (int tx) {
    	
    	
    	tileClass tile = tiles.get(tx);
    	if (tile.board_index >= 0) {
    		
    	
    	    BoardClass spot = board.get(tile.board_index);

	       // Log.d(TAG, "Removing tile from " + tile.board_index + " at "+ spot.getBoardCol() + 
	    	//		" " + spot.getBoardRow());
	    	spot.occupied = false;
	    	spot.setTileIndex(-1);
	    	for (int i=0; i<3; i++) {
		        spot.vertices[i] = 0;
		        
	    }
    	}
    	tile.setX((int)shelfZone.centerX());
	    tile.setY((int)shelfZone.centerY());
    	tile.played = false;
    	tile.inPlay = false;
    	tile.board_index = -1;
    	
    }
    
    /*
     * given index into the board space, bx, and an index in the tiles, tx
     * do all necessary to place the tile on the board.
     * return a positive score if this is a valid spot or -1 otherwise
     */
    private int placeTileOnBoard(int bx, int tx) {

    	Boolean matches[];
    	matches = new Boolean[4];
    	matches[LEFT] = false;
    	matches[TOP] = false;
    	matches[RIGHT] = false;
    	matches[BOTTOM] = false;
    	BoardClass spot = board.get(bx);
    	tileClass tile = tiles.get(tx);
    	               // set which tiles lives in this spot
    	spot.setTileIndex(tx);
	    tile.inPlay=true;
	    tile.played=true;
	    tile.setX(spot.getPixelX());
	    tile.setY(spot.getPixelY());
	    tile.board_index = bx;
   	
	    	          // copy vertices from the tile to the board. Once on the board, tiles don't move so
	                  // rotation is no longer a consideration.  Scores are as they are oriented on the tile
	    for (int i=0; i<3; i++) {
	        spot.vertices[i] = tile.getScore(i);
	    }
	    
	                  // first one gets points without having to check neighbors
	    if (TilesPlayed == 0) {
	    	int score = spot.vertices[0] + spot.vertices[1] + spot.vertices[2];
	    	return score;
	    }
	    else {
		    int score = -1; 
	        int face = 0;                        
		    int myRow = spot.getBoardRow();
			int myCol = spot.getBoardCol();
			             // check neighboring board spaces for valid placement.  There are maximum 3 neighboring tiles
			             // scoring is such that at least one face must match the adjacent face for a score to be valid.
			             // if this spot matches any one face, additional points my be earned. We'll get to those later.
			             // vertices are arranged as such:
			             //   \0   1 / 1 \
			             //    \ 2  / 0  2 \
	        
			                              
		    if (spot.isfaceDown()){         // a down facing tile has one face above, one on each side
		    	if (myCol > 0) {                // check left
				     BoardClass neighbor = board.get(ColRowGrid[myCol-1][myRow]);
				     if (neighbor.isOccupied()) { 
				    	 if ((spot.vertices[0] == neighbor.vertices[1]) &&
				             (spot.vertices[2] == neighbor.vertices[2])) {
				      		 face++;
				      		 matches[LEFT] = true;
				         }  else return -1;
				     }
		        }
	    		if (myRow > 0) {           //  check above
	    			 BoardClass neighbor = board.get(ColRowGrid[myCol][myRow-1]);
	    			 if (neighbor.isOccupied()) { 
	    				 if ((spot.vertices[0] == neighbor.vertices[0]) &&
	    				     (spot.vertices[1] == neighbor.vertices[2])) {
	    				     face++;
	    				     matches[TOP] = true;
	    				 } else return -1;
	    		    
	    			 }
	    		 }
	    		 if (myCol < mMaxCol-1) {        // check right
	    		     BoardClass neighbor = board.get(ColRowGrid[myCol+1][myRow]);
	    		     if (neighbor.isOccupied()) { 
	    			     if ((spot.vertices[1] == neighbor.vertices[1]) &&
	    			         (spot.vertices[2] == neighbor.vertices[0])) {
	    			          face++;
	    			          matches[RIGHT] = true;
	    			     } else  return -1;
	    			     
	    		     }
	    		       
	            }
	    		 
		    }
		    else {                         //  an up facing tile has one facing below and one on each side
		    	if (myCol > 0) {                // check left
				     BoardClass neighbor = board.get(ColRowGrid[myCol-1][myRow]);
				     if (neighbor.isOccupied()) { 
				    	 if ((spot.vertices[1] == neighbor.vertices[1]) &&
				             (spot.vertices[0] == neighbor.vertices[2])) {
				      		 face++;
				      		 matches[LEFT] = true;
				         } else return -1; 
				     }
		        }
		    	
		    	if (myRow < mMaxRow-1) {   // check below
	   			   BoardClass neighbor = board.get(ColRowGrid[myCol][myRow+1]);
	   			   if (neighbor.isOccupied()) { 
	   				   if ((spot.vertices[0] == neighbor.vertices[0]) &&
	   			           (spot.vertices[2] == neighbor.vertices[1])) {
	   					   face++;
				      	   matches[BOTTOM] = true;
	   					} else return -1; 
	   			   }
	   		    }
		    	if (myCol < mMaxCol-1) {        // check right
	   		     BoardClass neighbor = board.get(ColRowGrid[myCol+1][myRow]);
	   		     if (neighbor.isOccupied()) { 
	   			     if ((spot.vertices[1] == neighbor.vertices[0]) &&
	   			         (spot.vertices[2] == neighbor.vertices[2])) {
	   			          face++;
				      	  matches[RIGHT] = true;
	   			     }
	   			     else return -1;
	   		     }
	   		       
	           }
		    }
		    if (face == 0) {
		    	// sorry - no match
		    	return -1;
		    }
		    
		    if (face >= 2) {               // matched two solid faces. Golden! 
		    	                           // set score to 50 + sum and return
		    	score = 50 + spot.vertices[0] + spot.vertices[1] + spot.vertices[2];
		    	return score;
		    }
		    
		    if (face == 1) {
		    	score = spot.vertices[0] + spot.vertices[1] + spot.vertices[2];
		    	                           // we matched one edge of the triangle. simply add the vertices. but it's possible
		                                   // that we could match one edge and a single vertex - a bridge was formed
		                                   // or that a lone vertex throws off the matching altogether
			    if (spot.isfaceDown()){        
			    	if (matches[LEFT]) {
			    		// check c+2, r
			    		//       c+2, r-1
			    		//       c+1, r-1
			    		if (myCol < mMaxCol-2) {
			    			if (board.get(ColRowGrid[myCol+2][myRow]).isOccupied()) {
	 		   			       if (board.get(ColRowGrid[myCol+2][myRow]).vertices[0] == spot.vertices[1] ) {
	 		   			          return (score + 40);
	 		   			       }
	 		   			       else { 
	 		   			          return -1;
	 		   			       }
			    			}
			    		}
			    		if ((myCol < mMaxCol-2) && (myRow > 0)) {
			    			if (board.get(ColRowGrid[myCol+2][myRow-1]).isOccupied()) {
	 		   			       if (board.get(ColRowGrid[myCol+2][myRow-1]).vertices[0] == spot.vertices[1] ) {
	 		   			          return (score + 40);
	 		   			       }
	 		   			       else { 
	 		   			          return -1;
	 		   			       }
			    			}
			    		}
			    		if ((myCol < mMaxCol-1) && (myRow > 1)) {
			    			if (board.get(ColRowGrid[myCol+1][myRow-1]).isOccupied()) {
	 		   			       if (board.get(ColRowGrid[myCol+1][myRow-1]).vertices[2] == spot.vertices[1] ) {
	 		   			          return (score + 40);
	 		   			       }
	 		   			       else { 
	 		   			          return -1;
	 		   			       }
			    			}
			    		}
			    	}
			    	if (matches[RIGHT]) {
			    	    // check c-2, r
			    		//       c-2, r-1
			    		//       c-1, r-1
			    		if (myCol > 2 ) {
			    			if (board.get(ColRowGrid[myCol-2][myRow]).isOccupied()) {
		 		   			    if (board.get(ColRowGrid[myCol-2][myRow]).vertices[1] == spot.vertices[0] ) {
		 		   			       return (score + 40);
		 		   			    }
		 		   			    else { 
		 		   			       return -1;
		 		   			    }
			    			}
			    		}
		    			if ((myCol > 2 ) && (myRow > 1)) {
			    			if (board.get(ColRowGrid[myCol-2][myRow-1]).isOccupied()) {
		 		   			    if (board.get(ColRowGrid[myCol-2][myRow-1]).vertices[2] == spot.vertices[0] ) {
		 		   			       return (score + 40);
		 		   			    }
		 		   			    else { 
		 		   			       return -1;
		 		   			    }
			    			}
		    			}
			    	    if ((myCol > 1) && (myRow > 1)) {
			    	    	if (board.get(ColRowGrid[myCol-1][myRow-1]).isOccupied()) {
			    	    		if (board.get(ColRowGrid[myCol-1][myRow-1]).vertices[2] == spot.vertices[0] ) {
			    	    			return (score + 40);
			    	    		}
			    	    		else { 
			    	    			return -1;
			    	    		}
			    	    	}
			    	    }
			    	    
			    	}
			    	if (matches[TOP]) {
			    	    // check c-1, r+1
			    		//        c, r+1
			    		//       c+1, r+1
			    		if (myRow < mMaxRow-2) {
			    		   if (myCol > 1) {
	   		    		        if (board.get(ColRowGrid[myCol-1][myRow+1]).isOccupied()) {
		    	    		        if (board.get(ColRowGrid[myCol-1][myRow+1]).vertices[1] == spot.vertices[2] ) {
		    	    		  	        return (score + 40);
		    	    		        }
		    	    		        else { 
		    	    			        return -1;
		    	    		        }
	   		    		        }
			    		   }
			    	       if (myCol < mMaxCol-2) {
   		    		            if (board.get(ColRowGrid[myCol+1][myRow+1]).isOccupied()) {
	    	    		           if (board.get(ColRowGrid[myCol+1][myRow+1]).vertices[0] == spot.vertices[2] ) {
	    	    		  	           return (score + 40);
	    	    		           }
	    	    		           else { 
	    	    			           return -1;
	    	    		           }
   		    		            }
			    	       }
			    	       if (myRow < mMaxRow-1){
				    		   if (board.get(ColRowGrid[myCol][myRow+1]).isOccupied()) {
				    			  if (board.get(ColRowGrid[myCol][myRow+1]).vertices[1] == spot.vertices[2] ) {
				    				  return (score + 40);
				    			  }
				    			  else {
				    				  return -1;
				    			  }
				    		   }
			    	       }
			    		}
			    	    
			    	}
			    } 	
			    else { // this is a a face up tile
			    	
			    	if (matches[LEFT]) {
			    		// check c+2, r
			    		//       c+2, r+1
			    		//       c+1, r+1
			    		if (myCol < mMaxCol-2) {
			    			if (board.get(ColRowGrid[myCol+2][myRow]).isOccupied()) {
	 		   			       if (board.get(ColRowGrid[myCol+2][myRow]).vertices[0] == spot.vertices[2] ) {
	 		   			          return (score + 40);
	 		   			       }
	 		   			       else { 
	 		   			          return -1;
	 		   			       }
			    			}
			    		}
			    		if ((myCol < mMaxCol-2) && (myRow < mMaxRow-1)) {
			    			if (board.get(ColRowGrid[myCol+2][myRow+1]).isOccupied()) {
	 		   			       if (board.get(ColRowGrid[myCol+2][myRow+1]).vertices[0] == spot.vertices[2] ) {
	 		   			          return (score + 40);
	 		   			       }
	 		   			       else { 
	 		   			          return -1;
	 		   			       }
			    			}
			    		}
			    		if ((myCol < mMaxCol-1) && (myRow < mMaxRow-1)) {
			    			if (board.get(ColRowGrid[myCol+1][myRow+1]).isOccupied()) {
	 		   			       if (board.get(ColRowGrid[myCol+1][myRow+1]).vertices[1] == spot.vertices[2] ) {
	 		   			          return (score + 40);
	 		   			       }
	 		   			       else { 
	 		   			          return -1;
	 		   			       }
			    			}
			    		}
			    	}
			    	if (matches[RIGHT]) {
			    		 // check c-2, r
			    		//       c-2, r+1
			    		//       c-1, r+1
			    		if (myCol > 1 ) {
			    			if (board.get(ColRowGrid[myCol-2][myRow]).isOccupied()) {
		 		   			    if (board.get(ColRowGrid[myCol-2][myRow]).vertices[2] == spot.vertices[0] ) {
		 		   			       return (score + 40);
		 		   			    }
		 		   			    else { 
		 		   			       return -1;
		 		   			    }
			    			}
			    			if (myRow < mMaxRow-1) {
			    				if (board.get(ColRowGrid[myCol-2][myRow+1]).isOccupied()) {
				    	    		if (board.get(ColRowGrid[myCol-2][myRow+1]).vertices[1] == spot.vertices[0] ) {
				    	    			return (score + 40);
				    	    		}
				    	    		else { 
				    	    			return -1;
				    	    		}
			    				}
			    			}
			    		}
		    			if ((myCol > 1 ) && (myRow < mMaxRow-2)) {
			    			if (board.get(ColRowGrid[myCol-1][myRow+1]).isOccupied()) {
		 		   			    if (board.get(ColRowGrid[myCol-1][myRow+1]).vertices[1] == spot.vertices[0] ) {
		 		   			       return (score + 40);
		 		   			    }
		 		   			    else {
		 		   			       return -1;
		 		   			    }
			    			}
		    			}
			    	    
			    	}
			    	if (matches[BOTTOM]) {
			    		// check c-1, r-1
			    		//        c, r-1
			    		//       c+1, r-1
			    		if (myRow > 1) {
			    		   if (myCol > 1) {
	   		    		        if (board.get(ColRowGrid[myCol-1][myRow-1]).isOccupied()) {
		    	    		        if (board.get(ColRowGrid[myCol-1][myRow-1]).vertices[2] == spot.vertices[1] ) {
		    	    		  	        return (score + 40);
		    	    		        }
		    	    		        else { 
		    	    			        return -1;
		    	    		        }
	   		    		        }
			    		   }
			    	       if (myCol < mMaxCol-2){
   		    		            if (board.get(ColRowGrid[myCol+1][myRow-1]).isOccupied()) {
		    	    		        if (board.get(ColRowGrid[myCol+1][myRow-1]).vertices[0] == spot.vertices[1] ) {
		    	    		  	        return (score + 40);
		    	    		        }
		    	    		        else { 
		    	    			        return -1;
		    	    		        }
   		    		            }
			    	       }
			    		   if (board.get(ColRowGrid[myCol][myRow-1]).isOccupied()) {
			    			  if (board.get(ColRowGrid[myCol][myRow-1]).vertices[2] == spot.vertices[1] ) {
			    				  return (score + 40);
			    			  }
			    			  else {
			    				  return -1;
			    			  }
			    		   }
			    	    }
			    	}
			    }
		 
		    }

           return score;
       }
    }
    


    void InitBoardView(Context context) {
    	// get the display metrics
        screenMetrics = context.getResources().getDisplayMetrics();
		setFocusable(true); //necessary for getting the touch events
		controlTextSize = smTextSize;
		statusTextSize = vsmTextSize;
		int fudge = 10;

		// Use mGestureThreshold as a distance in pixels
		
		if (screenMetrics.heightPixels > 810) {
			BSIZE = 150;  // size of an background edge
		    TSIZE = 130;  // size of a tile edge
		    ALT = 150;
		    controlTextSize = lgTextSize;
		    statusTextSize = smTextSize;
		    fudge = 15;
		}
		else if ((screenMetrics.heightPixels > 400) && (screenMetrics.heightPixels < 600)) {
			BSIZE = 60;  // size of an background edge
		    TSIZE = 60;  // size of a tile edge
		    ALT = 60;
		}
		else if (screenMetrics.heightPixels < 400) {
			BSIZE = 40;  // size of an background edge
		    TSIZE = 35;  // size of a tile edge
		    ALT = 40;
		    fudge = 5;
		    statusTextSize = usmTextSize;
		    controlTextSize = usmTextSize;
		}
		else {
			BSIZE = 100;  // size of an background edge
		    TSIZE = 90;  // size of a tile edge
		    ALT = 100;
		}
        //ALT =  (int) (Math.sqrt(3f)/2f*BSIZE);		
		
		dialogUI = new ProgressDialog(mcontext);
		dialogUI.setMessage("Communicating...");  // I'll use this several times in the game
		
		
		
		BitmapFactory.Options options=new BitmapFactory.Options();
    	options.inSampleSize = 1;
    	options.inDither=true;                     //Disable Dithering mode
    	//options.inMutable = true;
    	options.inPurgeable=true;                   //Tell to gc that whether it needs free memory, the Bitmap can be cleared
    	options.inInputShareable=true;              //Which kind of reference will be used to recover the Bitmap data after being clear, when it will be used in the future
    	//options.inTempStorage=new byte[32 * 1024]; 
    	options.inPreferredConfig = Bitmap.Config.RGB_565;
        mcontext = context;
        sComm = new ServerCommunication(mcontext.getApplicationContext());
        localPlayPasses = 0;
        
        mMaxRow = mMapSize; 
    	mMaxCol = mMapSize;        // two columns per row since two triangles makes one square 		
    		 
    	TileExtents = new int[4];  // extents of the played tiles on the board
    	TileExtents[LEFT] = mMaxCol;
    	TileExtents[TOP] = mMaxRow;
    	TileExtents[RIGHT] = 0;
    	TileExtents[BOTTOM] = 0;
    	
    	Boolean success = false;
    	while (!success) {
    		try {
      	        background = Bitmap.createBitmap( (mMaxCol/2)*BSIZE+(BSIZE/2), mMaxRow*BSIZE, Bitmap.Config.RGB_565);
      	        success = true;
    		}  catch (OutOfMemoryError E) {
      	    	BSIZE -= 10;
      	    	TSIZE -= 10;
      	    	ALT -= 10;
      	    	Log.d(TAG,"Too much memory. Reducing footprint. BSIZE="+BSIZE);

      	    }
      	    finally {
      	    	Log.d(TAG,"Too much memory. giving up");
      	    }
      	        
    	}
    	Canvas c = new Canvas(background);
      
        
        //
    	// query the database and pull in game data
    	// 
		mDbHelper.getSettings();
		Cursor tc = mDbHelper.fetchGame(game_id);
    	if (tc.moveToFirst()) {
    	   user_id = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_USERID));
     	   userName = tc.getString(tc.getColumnIndex(mDbHelper.KEY_NAME));
 		   OppName = tc.getString(tc.getColumnIndex(mDbHelper.KEY_OPPNM)); 
 		   opp_id = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_OPP));
 		   userScore = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_SCORE));
 		   OppScore = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_OPPSC));
 		   tc.getInt(tc.getColumnIndex(mDbHelper.KEY_WHO));
 		}
 		tc.close();

        turnScore = 0;
        
        // set up background         
    	bgTileUp = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.tile_up,options);
    	bgTileDn = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.tile_dn,options);

     	int oldWidth = bgTileUp.getWidth();
		int oldHeight = bgTileUp.getHeight();
		int newWidth = BSIZE; // my code assumes they are square - so don't create otherwise
		int newHeight = BSIZE; 
		// calculate the scale
		float scaleWidth = ((float) newWidth) / oldWidth;
		float scaleHeight = ((float)newHeight) / oldHeight;

		// create a matrix for the manipulation
		Matrix matrix = new Matrix();
		// resize the bit map
		matrix.postScale(scaleWidth, scaleHeight);

		
		// recreate the new scaled Bitmap gameboard background
		Bitmap newbkg = Bitmap.createBitmap(bgTileUp, 0, 0,  oldWidth, oldHeight, matrix, true);	
		bgTileUp = newbkg.copy(Bitmap.Config.ARGB_8888, true);// recreate the new Bitmap
		newbkg = Bitmap.createBitmap(bgTileDn, 0, 0,  oldWidth, oldHeight, matrix, true);	
		bgTileDn = newbkg.copy(Bitmap.Config.ARGB_8888, true);
		
		// read in the buttons 
        play_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.play_button);
        pass_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.pass_button);
        recall_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.recall_button);
        cancel_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.cancel_button);
        submit_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.submit_button);
        check_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.check_button);
        chat_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.chat_icon);
        chat_new_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.messages);
        help_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.help_small);
        settings_button = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.settings_small);
        //
        //Bitmap question = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.question);
        //Bitmap redquestion = BitmapFactory.decodeResource(mcontext.getResources(), R.drawable.redquestion);

        
		
    	
    	
    	
    	// set up drawing zones of UI, shelf and game
    	uiZone = new RectF();
        gameZone = new RectF();    	    
    	shelfZone = new RectF(); 
    	tilesLeftZone = new RectF();
    	float fheight = (float) (screenMetrics.heightPixels-(0.09*screenMetrics.heightPixels)); // title bar consumes this much
    	gameZone.set(0, 0, screenMetrics.widthPixels, fheight*gamePct);
    	shelfZone.set(0, gameZone.bottom, screenMetrics.widthPixels, gameZone.bottom + (fheight*shelfPct));
    	uiZone.set(0, shelfZone.bottom, screenMetrics.widthPixels, shelfZone.bottom + (fheight*uiPct));
    	tilesLeftZone.set(gameZone.right-(2*TSIZE), 0, gameZone.right-TSIZE, TSIZE );
    	// within UI zone, set up button locations
    	int playX = 0;
    	int passX = (int) (0.20*uiZone.width());
    	int recallX = (int) (0.40 * uiZone.width());
    	int checkX = (int) (0.01 * uiZone.width()); // same as play
    	uiPlayBtn = new RectF(uiZone.left+playX,uiZone.top, uiZone.left+playX+play_button.getWidth(),uiZone.bottom);
    	uiPassBtn = new RectF(uiZone.left+passX,uiZone.top, uiZone.left+passX+pass_button.getWidth(),uiZone.bottom);
    	uiRecallBtn = new RectF(uiZone.left+recallX,uiZone.top, uiZone.left+recallX+recall_button.getWidth(),uiZone.bottom);
        uiCheckBtn = new RectF(uiZone.left+checkX,uiZone.top, uiZone.left+checkX+recall_button.getWidth(),uiZone.bottom);
        
        // top left of game is settings, help, chat
        int startLeft = 0;
        uiSettingsBtn = new RectF(0,0,settings_button.getWidth(),settings_button.getHeight());
        startLeft += settings_button.getWidth()+1;
        uiHelpBtn = new RectF(startLeft, 0 , startLeft + help_button.getWidth(),help_button.getHeight() );
        startLeft += help_button.getWidth();
    	uiChatBtn = new RectF(startLeft,0, startLeft + chat_button.getWidth(), chat_button.getHeight());
    	// subImage borders - start at center of large background subtracting 1/2 gameZone width and height
    	mStartRow = (int)((background.getHeight() / 2) - (gameZone.height()/2));
    	mStartCol = (int)((background.getWidth() / 2)  - (gameZone.width()/2));
    	
    	// setup virtual hex grid.  Each grid contains:
		// the centroid x,y screen locations (in pixels)
		// the virtualized x,y game board grid (in triangles)
		// the face style, up or down, occupied, etc. 
		//
    	float left = 0, top = 0;
    	float bgTileWidth = bgTileUp.getWidth();
    	float bgTileHeight = bgTileUp.getHeight();
    	float halfw = bgTileWidth/2;
    	int row = 0;
    	int col = 0;

    	Paint p = new Paint();
    	c.drawColor(GAME_BACKGROUND);
    	ColRowGrid = new int[mMaxCol][mMaxRow];  // initialize the col/row grid for easy access to triangular board spots
    	int count = 0;
    	left = halfw;
    	while (row < mMaxRow) {
    	   while (col < mMaxCol) {
               
    		   if ((row % 2)==0) {
    			   
	    	        // first do upward facing triangles on even number rows
    			    // on even rows, upward facing ones are shifted 1/2 triangle left
		   	        BoardClass h = new BoardClass();
			        h.setPixelLoc ( ((left-halfw) + left + (halfw+left)) / 3,    // bottom left, midpt, bottom right 
			                        (((ALT + ALT)*(row+1) + top) / 3)-fudge);        // bottom left*2, apex
			        h.setBoardLoc(col, row);
			        h.facedown = false;
			        h.index = count;
			        ColRowGrid[col][row] = count;
			        	            
			        board.add(count,h); 
			        
			        // draw the bitmap onto the canvas
			        c.drawBitmap(bgTileUp, left-halfw, top, null);
			        //if ((row%6==0) && (col%6==0)){
		            //	   c.drawBitmap(redquestion, h.getPixelX()-question.getWidth()/2, 
		            //			   h.getPixelY()-question.getHeight()/2, null);
		            //}
			        col++;
			        count++;
	    		   
			        // now do downward facing triangles
	    	        BoardClass k = new BoardClass();
	    	        k.setPixelLoc ( (left + (left+(halfw) + (left+bgTileWidth))) / 3, // top left,  midpt, top right
	    	        		        (fudge+(top + ALT *(row+1) + top) / 3));                             
	    	        k.setBoardLoc(col, row);
	    	        k.facedown = true;
	    	        k.index = count;
	    	        ColRowGrid[col][row] = count;
		            
	    	        board.add(count,k); 
	    	        // draw the bitmap onto the canvas
			        c.drawBitmap(bgTileDn, left, top, null);
			        //if ((row%6==0) && (col%6==0)){
			        //	c.drawBitmap(question, k.getPixelX()-question.getWidth()/2, 
		            //			   k.getPixelY()-question.getHeight()/2, null);
		            //}
	    	        count++;
	    	        col++;
			        left += bgTileWidth;
    		   }
    		   else { // on odd-number rows, do the down facing triangles first
    			    // first do downward facing triangles, this time the down facing are shifted 1/2
    			    // of a triangle to the left

	    	        BoardClass k = new BoardClass();
	    	        k.setPixelLoc ( ((left-halfw) + left + (left+halfw))/3, // top left,  midpt, top right
	    	        		        ((top + ALT *(row+1) + top) / 3)+fudge);                             
	    	        k.setBoardLoc(col, row);
	    	        k.facedown = true;
	    	        k.index = count;
	    	        ColRowGrid[col][row] = count;

	    	        board.add(count,k);
	    	        // draw the bitmap onto the canvas
			        c.drawBitmap(bgTileDn, left-halfw, top, null);
			        //if ((row%5==0) && (col%5==0)){
			        //	c.drawBitmap(question, k.getPixelX()-question.getWidth()/2, 
		            //			   k.getPixelY()-question.getHeight()/2, null);	
			        //	}
	    	        count++;
	    	        col++;
	    	        
	    	        
	    	        // now do upward facing triangles on odd number rows
		   	        BoardClass h = new BoardClass();
			        h.setPixelLoc ( ( left + (left+halfw) + (left+bgTileWidth)) / 3,    // bottom left, midpt, bottom right 
			                        (((ALT + ALT)*(row+1) + top) / 3)-fudge);            // bottom left*2, apex
			        h.setBoardLoc(col, row);
			        h.facedown = false;
			        h.index = count;
			        ColRowGrid[col][row] = count;

			        board.add(count,h); 
			        // draw the bitmap onto the canvas
			        c.drawBitmap(bgTileUp, left, top, null);
			        //if ((row%5==0) && (col%5==0)){
			        //	c.drawBitmap(question, h.getPixelX()-question.getWidth()/2, 
		            //			   h.getPixelY()-question.getHeight()/2, null);
			        //	}
			        col++;
			        count++;
			        left += bgTileWidth;
			        
    		   }
    		   
    	    }
    	   
    	  
    	    row++;
    	    top += ALT; // adjust by altitude of triangle
    	    

    	    left = halfw;
    	    col=0;
    	}
    	
    	// draw start circle
    	int x = mMaxCol / 2;
    	int y = mMaxRow / 2;
    	BoardClass spot = board.get(ColRowGrid[x][y]);
    	p.setColor(Color.RED); 
    	c.drawCircle(spot.getPixelX(), spot.getPixelY(), 10, p);
        p.setColor(Color.BLACK);
        String status = "Start Here";
    	c.drawText(status, spot.getPixelX()-(p.measureText(status)), spot.getPixelY(), p);
    	int textWidth = (int)p.measureText(status);    	
    	  // set up tiles. 
    	tileClass tiley = new tileClass(context, TSIZE, (int)(0.1667 * TSIZE));
        count = 0;                
        for (tileId tile: tileId.values()) {
        	String ID = tile.toString();
        	
        	Point point = new Point();
        	// put them initially on the shelf on the bottom
			int X = 0;
			int Y = (int) (shelfZone.bottom -TSIZE);
			point.x =  X;
			point.y =  Y;
			tiles.add( new tileClass(context, count, ID,  point, TSIZE, (int)(0.1667 * TSIZE)));
			count++;
        }
    	
        refreshUsersTiles();
        
        // query the database for tiles that have already been played
        refreshPlayedTiles();
        
        if (mDbHelper.isPlayingDevice(game_id)) {  // user is playing just the computer
        	refreshDeviceTiles();
        }

        madePlay = false;

        int centerX = board.get(ColRowGrid[mMaxCol/2][mMaxRow/2]).getPixelX();
        int centerY = board.get(ColRowGrid[mMaxCol/2][mMaxRow/2]).getPixelY();

        mStartCol =  centerX - (int)(gameZone.width()/2);
        mStartRow =  centerY - (int)(gameZone.height()/2);
        mXOffset = mStartCol;
        mYOffset = mStartRow;
        currentTile = -1;
        mid = new PointF();
        bgTileUp.recycle();
        bgTileDn.recycle();
        
        // check for any messages between users
    	
    	dialogUI.show();
    	Thread checkForChats = new Thread() {  
    	    public void run() {

    	    	Looper.prepare();
		        try {
					num_messages = sComm.checkChat(game_id, user_id);
				} catch (JSONException e) {
					e.printStackTrace();
				}
		        
			    	handler.sendEmptyMessage(0);
			    }
		};
		checkForChats.start();
    	 
        
        
    }

    void refreshDeviceTiles () {
    	//
    	// query the database and pull in device's tiles
    	// 
    	int count = 0;
    	Cursor tc = mDbHelper.fetchDeviceTiles(game_id);
    	if (tc.getCount() > 0){
    		deviceHand = new int [tc.getCount()];
			tc.moveToFirst();
			do {
				deviceHand[count] = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_TILE));
				count++;
			} while (tc.moveToNext());
		}
    	tc.close();
		deviceHandNumber = count;
		
		for (int i=0; i<deviceHandNumber; i++) {
        	
        	//int Xshift = (TSIZE/3) + (int) ((shelfZone.width()-(TSIZE/3))/count) * i ;
        	
        	tiles.get(deviceHand[i]).setX((TSIZE*i) + (TSIZE/2));
        	tiles.get(deviceHand[i]).setY((int)shelfZone.top-(TSIZE/2));
        	//if (!tiles.get(deviceHand[i]).isInPlay()) {
	        //	if ((i % 2) == 0) {
	        //		tiles.get(deviceHand[i]).setRotate(3,true);
	        //	}
	        //	else {
	        //		tiles.get(deviceHand[i]).setRotate(0,true);
	        //	}
        	//}
        }

    }
    void refreshUsersTiles () {
    	//
    	// query the database and pull in user's tiles
    	// 
    	int count = 0;
    	Cursor tc = mDbHelper.fetchMyTiles(game_id);
    	if (tc.getCount() > 0){
    		myHand = new int [tc.getCount()];
			tc.moveToFirst();
			do {
				myHand[count] = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_TILE));
				count++;
			} while (tc.moveToNext());
		}
    	tc.close();
		myHandNumber = count;
        for (int i=0; i<myHandNumber; i++) {
        	tiles.get(myHand[i]).inPlay = false;
        	int Xshift = (TSIZE/3) + (int) ((shelfZone.width()-(TSIZE/3))/count) * i ;
        	
        	tiles.get(myHand[i]).setX((TSIZE/4) + Xshift);
        	tiles.get(myHand[i]).setY((int)shelfZone.top+(TSIZE/2));
        	if ((i % 2) == 0) {
        		tiles.get(myHand[i]).setRotate(3,true);
        	}
        	else {
        		tiles.get(myHand[i]).setRotate(0,true);
        	}
        }
    	
    }

    
    private void refreshPlayedTiles () {

    	

    	int minX = 20;
    	int maxX = 0;
    	int minY = 20;
    	int maxY = 0;
		Cursor tc = mDbHelper.fetchPlayedTiles (game_id);   
    	if (tc.getCount() > 0){
			tc.moveToFirst();
			do {
				
				int index = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_TILE));

				if (index >= 0) {
					TilesPlayed++;
					tileClass tile = tiles.get(index);
					//tile.facedown = ( tc.getInt(tc.getColumnIndex(mDbHelper.KEY_FACEDN)) == 1);
					tile.setRotate( tc.getInt(tc.getColumnIndex(mDbHelper.KEY_ROT)), true );
					int tcol = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_COL));
					int trow = tc.getInt(tc.getColumnIndex(mDbHelper.KEY_ROW));
					
					
					if (tcol < minX) {
						minX = tcol;
					}
					if (tcol > maxX) {
						maxX=tcol;
					}
					if (trow < minY) {
						minY=trow;
					}
					if (trow > maxY) {
						maxY=trow;
					}
				
					int bx = ColRowGrid[tcol][trow];
					board.get(bx).setTileIndex(index);
					board.get(bx).occupied = true;
				                   // set this tile to be "played" and set it's location is on the board
				    tile.played = true; 
				    tile.setX(board.get(bx).getPixelX());
				    tile.setY(board.get(bx).getPixelY());
				    tile.board_index = bx;
				    	
				    	          // copy vertices from the tile to the board. Once on the board, tiles don't move so
				                  // rotation is no longer a consideration.  Scores are as they are oriented on the tile
				    for (int i=0; i<3; i++) {
				    	board.get(bx).vertices[i] = tile.getScore(i);
				    }
				}
			} while (tc.moveToNext());
		}
    	TileExtents[LEFT  ] = (minX-2 < 0)?0:minX-2;
    	TileExtents[RIGHT ] = (maxX+2 > mMaxCol-1)?mMaxCol-1:maxX+2;
    	TileExtents[TOP   ] = (minY-2 < 0)?0:minY-2;
    	TileExtents[BOTTOM] = (maxY+2 > mMaxRow-1)?mMaxRow-1:maxY+2;
    	Log.d(TAG,"Extents LR TB: "+
    			TileExtents[LEFT  ]+" " +
    			TileExtents[RIGHT ]+" " +
    			TileExtents[TOP   ]+" " +
    			TileExtents[BOTTOM]);
        tc.close();
        
    }
    
    public void game_over() {
    	SoundManager.playSound(SoundManager.Sounds.game_winner.sound(), 1);
     	mDbHelper.setGameOver(game_id); // local DB
     	
     	dialogUI.show();
    	Thread SetGameOver = new Thread() {  
    	    public void run() {

    	    	Looper.prepare();
    	    	sComm.setGameOver(game_id); // remote Server DB
    	    	if (mDbHelper.isPlayingDevice(game_id)) {
    	    		sComm.setFinalScore(game_id, userScore, OppScore); 
    	    	}
			  	handler.sendEmptyMessage(0);
			}
		};
		SetGameOver.start();
    	
    	
    }
    
    public void askRematch() {
    	if (!justBrowsing) {  // game over, user already said no rematch
	    	String message = "";
	    	justBrowsing = true;
	    	
	    	if (mDbHelper.isSoloGame(game_id)) {
	    		if (mDbHelper.tilesRemaining(game_id)) 
	    			message = "You Won!";
	    		else 
	    			message = "You Lost!";
	    	}
		    else {
		    	
		    	if (userScore > OppScore) {
		    		message = "You beat " + OppName;
		        }
		    	else {
		    		message = OppName + " beat you.";
		    	}
		    
	       	    if (userScore == OppScore) {
	    			message = "Its a Draw";
	    		}
		    }
	    	
	    	
	    	AlertDialog.Builder builder = new AlertDialog.Builder(this.getContext());
	    	if ((mDbHelper.isSoloGame(game_id)) || (mDbHelper.isPlayingDevice(game_id)) ) {
	    		builder.setMessage(message).setPositiveButton("Ok", dialogOkListener)
		    	    .show();
	    		
	    	}
	    	else {
	    		message += " Would you like a rematch?";
	    	    builder.setMessage(message).setPositiveButton("Yes", dialogClickListener)
	    	    .setNegativeButton("No", dialogClickListener).show();
	    	}
    	    
    	}
    }

    DialogInterface.OnClickListener dialogOkListener = new DialogInterface.OnClickListener() {
	    public void onClick(DialogInterface dialog, int which) {
	    	// do nothing
	    }
    };
    
	DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
	    public void onClick(DialogInterface dialog, int which) {
	        switch (which){
	        case DialogInterface.BUTTON_POSITIVE:
	            //Yes button clicked 
	        	// create new game 
	        	
	        	dialogUI.show();
	        	Thread newChallenge = new Thread() {  
				    public void run() {

				    	Looper.prepare();
			        	try {
							long game_id = sComm.gameChallenge(userName, user_id, OppName);
							if (game_id >0) {
								challengeMade = true;
							}
						} catch (JSONException e) {
							Toast.makeText(mcontext, "Error in Communication. Try Again", Toast.LENGTH_SHORT).show();
							e.printStackTrace();
						}
			        	handler.sendEmptyMessage(0);
					}
				};
			    newChallenge.start();
				
	        	
	            break;
	            

	        case DialogInterface.BUTTON_NEGATIVE:
	        	Log.d(TAG,"game finished" );
	        	//((Activity)getContext()).finish();
	        	// instead of exiting, allow user to stay and look
	            break;
	        }
	    }
	};
	
	final Handler handler = new Handler() {
	    public void handleMessage(Message msg) {
	    	Log.d(TAG,"handleMessage called");
	    	madePlay = true;
			dialogUI.dismiss();
			if (challengeMade) {
				challengeMade = false;
				((Activity)getContext()).finish(); // finish this game, go back to SelectGames
			}
		}
	};
	
    public void onDestroy() {
    	Log.d(TAG, "OnDestroy entered");
    	background.recycle();
    	bgTileUp.recycle();
    	bgTileDn.recycle();
    	play_button.recycle();
        pass_button.recycle();
        recall_button.recycle();
        cancel_button.recycle();
        submit_button.recycle();
    	
   		tiles.removeAll(tiles);
   		tiles.removeAll(board);
   		mDbHelper.close();
    	
    	
    	
    }
/*
    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
    	super.onMeasure(widthMeasureSpec, heightMeasureSpec);

        final int widthMode = MeasureSpec.getMode(widthMeasureSpec);
        if (widthMode != MeasureSpec.EXACTLY) {
            throw new IllegalStateException("Workspace can only be used in EXACTLY mode.");
        }

        final int heightMode = MeasureSpec.getMode(heightMeasureSpec);
        if (heightMode != MeasureSpec.EXACTLY) {
            throw new IllegalStateException("Workspace can only be used in EXACTLY mode.");
        }
        setMeasuredDimension(600, 800);
        //getChildAt(0).measure(widthMeasureSpec, heightMeasureSpec);
        super.onMeasure(MeasureSpec.makeMeasureSpec(480, MeasureSpec.EXACTLY), 
		        MeasureSpec.makeMeasureSpec(800, MeasureSpec.EXACTLY));
    
    }
 */
 /* 
    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        final View child = getChildAt(0);
        child.layout(0, 0, child.getMeasuredWidth(), child.getMeasuredHeight());
    }
*/
}
