package com.ulsanonline.triominoes;

/*
 * 02/25/2012 - Initial release
 * 02/29/2012 - v1.003 - findChallenges must be called by with notify=false
 *                       so we only create games when challenges are found
 *                       and we are in UI
 *                       streamline checkplay to look at only their turn games
 *                       split game list into my turn/their turn
 * 03/10/2012 - v2.0     allow user to play solo games
 * 09/30/2012 - v3.4     Added button to rate product on Google Play
 * 11/22/2012 - v3.5     improved threading for network access
 * 11/30/2012 - v3.6     fixed bug in thread - toast -> log.d
 * 12/02/2012 - v3.7     fixed bug if screen rotation while dialog
 * 12/23/2012 - v3.9     Improved multithreading behavior
 * 04/09/2013 - V4.2     re-instate ratings button
 * 09/28/2013 - v4.3     cleaned up games list.  Rotate tile as in game. Added progressbar to start
 * 						 and added threading where needed for network traffic method calls
 *                     
 */


import java.text.SimpleDateFormat;
import java.util.Calendar;
import android.text.format.DateUtils;
import android.util.Log;

import org.json.JSONException;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.NotificationManager;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.graphics.Point;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;

public class SelectGames extends Activity {
	
	private triDbAdapter mDbHelper;
	private ServerCommunication sComm;
	private long delete_game_id = -1;
	private MyDataAdapter myGameAdp;
	private MyDataAdapter theirGameAdp;
	private String userName;
	private boolean challengesExist;
	private boolean updatesExist;
	private int user_id;
	private Calendar cal;
	private static final int tSize = 100;
	SimpleDateFormat sdf;
	String currentDateString;
    private ListView myTurn;
    private ListView theirTurn;
    private LinearLayout myTurnLL;
    private LinearLayout theirTurnLL;
	private ProgressDialog pDialog; 
    private AlertDialog.Builder dialogBuilder;
    private Boolean doChecks;
    private String TAG="SelectGames";
	
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        cal = Calendar.getInstance();
        setContentView(R.layout.games_list);
        setTitle("My Games");
        String ns = SelectGames.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) getSystemService(ns);
		mNotificationManager.cancelAll();
        
		pDialog = new ProgressDialog(SelectGames.this);
		
		// do communication checks with server on the first time
		doChecks = true;
		
		// unless... there's a bundle with this Intent, check for
		// coming with a NO_CHECK flag. 
		Bundle extras = getIntent().getExtras();
        if (extras != null) { // check for null and terminate
            doChecks =  (extras.getInt("NO_CHECK") != 1);
        }
		
		
        mDbHelper = new triDbAdapter(this.getApplicationContext());
        //mDbHelper.open();
        
        sComm = new ServerCommunication(this);
       

        myTurn = (ListView)findViewById(R.id.ListView1);
        myTurnLL = (LinearLayout)findViewById(R.id.mine);
        theirTurn = (ListView)findViewById(R.id.ListView2);
        theirTurnLL = (LinearLayout)findViewById(R.id.theirs);
        
        dialogBuilder = new AlertDialog.Builder(SelectGames.this);
        
	}
	
	private void populateData() {
        String[] from;
        int[] to;
        //Log.d(TAG, "open on populateData");
        //mDbHelper.open();
		mDbHelper.getSettings();
		userName = mDbHelper.getName();
		user_id = mDbHelper.getUserId();
		   
		Cursor c = mDbHelper.fetchAllGamesInPlay(mDbHelper.getUserId(),"<>");   // myturn = games with KEY_WHO <> myid
        startManagingCursor(c);
        Log.d(TAG, "My Turn="+c.getCount());
        
        Cursor c2 = mDbHelper.fetchAllGamesInPlay(mDbHelper.getUserId(),"=");   // theirturn = games with KEY_WHO = myid
        startManagingCursor(c2);
        Log.d(TAG, "Their Turn="+c2.getCount());
        
        from = new String[] {triDbAdapter.KEY_ID, 
        	                 triDbAdapter.KEY_OPP, 
        	                 triDbAdapter.KEY_OPPSC, 
        	                 triDbAdapter.KEY_OPPNM,
        	                 triDbAdapter.KEY_DATE,
        	                 triDbAdapter.KEY_LASTN,
        	                 triDbAdapter.KEY_LASTT,
        	                 triDbAdapter.KEY_TSCORE,
        	                 triDbAdapter.KEY_WHO,
        	                 triDbAdapter.KEY_COMPLETE};
        	                 
        to = new int []  {R.id.game_id,
        		          R.id.oppname,
        		          R.id.oppscore,
        		          R.id.date}; 
        
        myGameAdp = new MyDataAdapter (this, R.layout.show_games, c, from, to);
        myTurn.setAdapter (myGameAdp);
        theirGameAdp = new MyDataAdapter (this, R.layout.show_games, c2, from, to);
        theirTurn.setAdapter (theirGameAdp);
        
	}
	

	// server access requires work off the main UI thread.  Setup aynctask			
	private void doServerCommunication () {
		pDialog.setMessage("Check remote player moves...");  
		pDialog.show();

		challengesExist = false;
		updatesExist = false;
    	
    	new AsyncTask<Void,Void, Boolean>(){
            @Override
            protected void onPostExecute(Boolean found) {            	
            	pDialog.dismiss();
            	
            	
            	if (challengesExist) {
        			// hey - someone challenged me! Dialogue
        	        // if user wants to play, accept this game
        			// or delete it.
        			String name = sComm.getName(); // this is the challenger's name
        			
        			dialogBuilder.setMessage("You have been challenged by "+name).setPositiveButton("Accept", ChallengeClickListener)
        			    .setNegativeButton("Decline", ChallengeClickListener).show();
        		}
            	if (updatesExist) {
            		// someone made a move. Re-populate the views to show what's happened
        	       populateData();
	        	   buildListViews();
            	}

            }
            
            @Override
			protected Boolean doInBackground(Void... params) {
            	boolean status = false;
						
                // has anyone challenged this user?	
				try {
					challengesExist =  sComm.findchallenges(userName, user_id, false);
				 	if (challengesExist) {
				 		sComm.gameAccept(sComm.getId(), user_id, sComm.getGame(), userName, true);
				 	}
				} 
				catch (JSONException e) {
					e.printStackTrace();
				}
				catch (ClassCastException e) {
		    		Log.d(TAG, "Communication Error. Check your internet connection");
					e.printStackTrace();
				}
				
				// has a remote user made a play?
				updatesExist = checkplay();
				
				return updatesExist;
            }
            
        }.execute();
    	
	}
				


	// check for any remote plays made by another user
	private boolean checkplay() {
		boolean updates = false;
		Cursor c;

		c = mDbHelper.fetchAllGamesInPlay(mDbHelper.getUserId(),"=");   // just those games with KEY_WHO = this user
    	if (c.getCount() > 0){
			c.moveToFirst();
			do {
				//if (c.getInt(c.getColumnIndex(mDbHelper.KEY_COMPLETE))==0) { // dont check completed
					 long game_id = c.getInt(c.getColumnIndex(mDbHelper.KEY_ID));
					
					
		                try {
							if ( sComm.checkplay(game_id, 0) ) { 
		                        updates = true;
							}
						} 
		                catch (JSONException e) {
							//c.close();
							e.printStackTrace();
						}
			                
				//}
			} while (c.moveToNext());
		}
    	c.close();

    	return updates;
    	
	}
	
	
	
	// only show the listview where games exist. Don't bother showing a listview header where none exist
	private void buildListViews() {
		Cursor c = myGameAdp.getCursor();
  		if (c.getCount()==0) 
        	myTurnLL.setVisibility(View.GONE);
        else {
        	myTurnLL.setVisibility(View.VISIBLE);
      		myGameAdp.notifyDataSetChanged();
        }
        	
  		c = theirGameAdp.getCursor();
  		
  		if (c.getCount()==0) 
        	theirTurnLL.setVisibility(View.GONE);
        else {
        	theirGameAdp.notifyDataSetChanged();
        	theirTurnLL.setVisibility(View.VISIBLE);
        }
        	theirTurnLL.setVisibility(View.VISIBLE);
		
	}
	
	// Will be called via the onClick attribute
	// of the buttons in main.xml
	public void onClick(View view) {
		//@SuppressWarnings("unchecked")
	
		switch (view.getId()) {
		
		          // "create new game" button pressed. start activity
		case R.id.begin:
		         Intent i = new Intent(this,ChallengeUser.class);
				 startActivity(i);
			break;

			      // refresh button pressed. check for remote actions
		case R.id.refresh:
			 
			    doServerCommunication();
				
			break;
			
			      // rate button pressed - I hope they like me.
		case R.id.rate:
				Intent intent = new Intent(Intent.ACTION_VIEW);
		        intent.setData(Uri.parse("market://details?id=com.ulsanonline.triominoes"));
		        startActivity(intent);
			break;
		}

	}

	
	// this dialog asks user to delete a game or not
	DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
	    public void onClick(DialogInterface dialog, int which) {
	        switch (which){
	        case DialogInterface.BUTTON_POSITIVE:
	            //Yes button clicked - accept the game
	        	mDbHelper.open();
	        	mDbHelper.DeleteGame(delete_game_id);

	        	new AsyncTask<Void,Void, Boolean>(){
	        		
	        		protected void onPreExecute() {
	        			pDialog.setMessage(""); 
	    	        	pDialog.show();
	        		};

	                @Override
	    			protected Boolean doInBackground(Void... params) {

				      String err = sComm.setGameOver(delete_game_id);
	                  return (err.contains("Error"));
	                }
	                
	                @Override
	                protected void onPostExecute(Boolean found) {            	
	                	pDialog.dismiss();
	                	
	                	Cursor c = myGameAdp.getCursor();
	                	c.requery();
	                	Cursor c2 = theirGameAdp.getCursor();
	                	c2.requery();
	                	buildListViews();
	    	        	mDbHelper.close();

	                }
	                
	            }.execute();


	        	
	            break;

	        case DialogInterface.BUTTON_NEGATIVE:
	            // then keep it
	            break;
	        }
	    }
	};
	
	
	// this dialog is to accept or decline a new game challenge
	DialogInterface.OnClickListener ChallengeClickListener = new DialogInterface.OnClickListener() {
	    public void onClick(DialogInterface dialog, int which) {
	        switch (which){
	        case DialogInterface.BUTTON_POSITIVE:
	            //Yes button clicked - accept the game
	        	challengesExist = true;
	        	populateData();
	        	buildListViews();
	            break;

	        case DialogInterface.BUTTON_NEGATIVE:
	            //No button clicked - delete the game
	        	mDbHelper.open();
	        	mDbHelper.DeleteGame(sComm.chGameID);

	        	
	        	pDialog.setMessage(""); 
	        	pDialog.show();
			    Thread gameOver = new Thread() {  
			        public void run() {

		   		    	Looper.prepare();
				        sComm.deleteChallenge(sComm.chGameID, user_id);
						
				    	handler.sendEmptyMessage(0);
				    }
				};
				gameOver.start();
	        	
	        	
	        	
	        	mDbHelper.close();
	        	myGameAdp.getCursor();
	        	myGameAdp.notifyDataSetChanged();
	            break;
	        }
	    }
	};

	
	// when thread has finished, just kill the dialog
	final Handler handler = new Handler() {
	    public void handleMessage(Message msg) {
			pDialog.dismiss();
		}
	};
	

	// this adapter builds a game status into a listview for each of the games currently in the DB
	public class MyDataAdapter extends SimpleCursorAdapter {
		private Cursor c;
		private Context context;
		
	    public MyDataAdapter(Context context, int layout, Cursor c, String[] from, int[] to) {
	    	super(context, layout, c, from, to);
	    	this.c = c;
		    this.context = context;
	        tileClass tiley = new tileClass(context, tSize, (int)(0.2 * tSize));
	    
	    }
    	
	    	
    	public View getView(final int pos, View inView, ViewGroup parent) {
			View vi=inView;
			 TextView textname;
			 TextView textscore;
			 TextView textdate;
			 TextView textid;
			 final ProgressBar bar;
			 ImageView image;
	        
	        if (inView == null) {
	     	    LayoutInflater inflater = (LayoutInflater) context
		                .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		        vi = inflater.inflate(R.layout.show_games, null);
		        
                
                //vi.setTag(holder);

		    }
	        textname=(TextView)vi.findViewById(R.id.oppname);
            textscore=(TextView)vi.findViewById(R.id.oppscore);
            textdate=(TextView)vi.findViewById(R.id.date);
            textid=(TextView)vi.findViewById(R.id.game_id);
            image=(ImageView)vi.findViewById(R.id.icon);
            bar = (ProgressBar)vi.findViewById(R.id.progessBar1);
            bar.setVisibility(View.GONE);
	        
	        //all your inflation and setting values goes here and at the end
	        //add listener to the button also and also to the row view

	        
	        
	        c.moveToPosition(pos);
	        int who = c.getInt(c.getColumnIndex(triDbAdapter.KEY_WHO));
            int opponent= c.getInt(c.getColumnIndex(triDbAdapter.KEY_OPP));
            int opScore = c.getInt(c.getColumnIndex(triDbAdapter.KEY_OPPSC));
            int myScore = c.getInt(c.getColumnIndex(triDbAdapter.KEY_SCORE));
            String oppName = c.getString(c.getColumnIndex(triDbAdapter.KEY_OPPNM));            
            int tScore = c.getInt(c.getColumnIndex(triDbAdapter.KEY_TSCORE));
	        int complete = c.getInt(c.getColumnIndex(triDbAdapter.KEY_COMPLETE));
	        String done = "";
	        if (opponent == -1) {   // solo game
	        	done = "Solo Game: You played for " + tScore + " points";
	        	textname.setText(done);
	        } 
	        else {
		        if (complete == 1) {
		        	if (opScore > myScore) 
		        		done = oppName + " beat you " + opScore + "-" + myScore;
		        	else
		        		done = "You beat " + oppName + " " + myScore + "-" + opScore;
		        	textname.setText(done);
		        }
		        else {
		        	if (who == opponent) 
		        		done = oppName + " played ";
		        	else
		        		done = "You played vs. " + oppName + " ";
		        	textname.setText(done + "for " + tScore + " points");
		        }
	        }

            String start = (complete==1)?"COMPLETED " : "Last played  ";
            String date = (String) DateUtils.getRelativeTimeSpanString (c.getLong(
            		               c.getColumnIndex(triDbAdapter.KEY_DATE)));
            textdate.setText(start + date);
            String game_id = c.getString(c.getColumnIndex(triDbAdapter.KEY_ID));
            textid.setText(game_id);
            
            
            int num = c.getInt(c.getColumnIndex(triDbAdapter.KEY_LASTT));
            if (num != -1) {
	            String fname = c.getString(c.getColumnIndex(triDbAdapter.KEY_LASTN));
	            int rot =      c.getInt(c.getColumnIndex(triDbAdapter.KEY_ROT));
	            Point p = new Point();
	            p.x=0; p.y=0;
	            tileClass tile = new tileClass(context, num, fname, p, tSize, (int)(0.2 * tSize));
	            tile.setRotate(rot, true);
	            image.setImageBitmap(tile.getBitmap());
            }
            else {
            	image.setImageResource(R.drawable.pass_button);
            }
            
	        vi.setOnClickListener( new View.OnClickListener()
	        {
	                public void onClick(View v)
	                {
	                	TextView textid=(TextView)v.findViewById(R.id.game_id);
	                	bar.setVisibility(View.VISIBLE);
	                	long game_id = Integer.parseInt(textid.getText().toString());
	                	Bundle bundle = new Bundle();
			        	bundle.putLong(triDbAdapter.KEY_ID, game_id);
	        			Intent i = new Intent(v.getContext(), TriOminoesStartActivity.class);
						i.putExtras(bundle); 
	        			startActivity(i);
	                }
	        });
	        
	        vi.setOnLongClickListener( new View.OnLongClickListener()
	        {
	                public boolean onLongClick(View v)
	                {
	                	TextView textid=(TextView)v.findViewById(R.id.game_id);
	                	long game_id = Long.parseLong(textid.getText().toString());
	                	delete_game_id = game_id;
	                	dialogBuilder = new AlertDialog.Builder(v.getContext());
	                	dialogBuilder.setMessage("Do you really want to delete this game?").setPositiveButton("Yes", dialogClickListener)
	                	    .setNegativeButton("No", dialogClickListener).show();
	                	
	                	return true;
	                }
	        });
           
	        return vi;
    	}
    	
    	
	}
	
	
	@Override
	protected void onResume() {
		super.onResume();
		Log.d(TAG, "open on resume");
		mDbHelper.open();
		if (doChecks) {
		   doServerCommunication();
		}

		populateData();
		buildListViews();
	    
	}
	
	@Override
	protected void onRestart() {
		super.onRestart();

		doChecks = false; // come back in again, I'd like NOT to do checks 

	    
	}
	

	@Override
	protected void onPause() {
		super.onPause();
		Log.d(TAG, "close onPause");
		mDbHelper.close();
		
	}
	@Override
	public void onDestroy() {
		super.onDestroy();
		Log.d(TAG, "close onDestroy");
		mDbHelper.close();

	}
}
