package com.ulsanonline.triominoes;
/*
 * 02/25/2012 - Initial release
 * 02/29/2012 - v1.003 - added method deleteuser() to communicate with php server on demise of user
 *                     - findChallenges may now be called by game or by WakeCheck for notifications
 *                     - make sure checkplay() is called after challenge is found. 
 * 03/10/2012 - v2.0   - allow the user to play a solo game
 * 03/11/2012 - v3.0   - enable in-game chats.  Added methods to send, recv and check for chats
 * 03/18/2012 - v3.1   - play against the Android computer
 * 12/22/2012 - v3.9   - game_id: int -> long
 * 01/28/2013 - v4.1   - make sure when remote player completes game local player sees it
 *                     - check for completed game on remote move before checking for play move
 *
 *                     
 */
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import android.content.Context;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Build;
import android.util.Log;


public class ServerCommunication  {

	public static final String NO_CHALLENGE ="_none";
	public static String chPlayer2name;
	public static int chPlayer2_ID;
	public static long chGameID;
    private triDbAdapter mDbHelper;
    
    private static String server = "http://martypants.us/";
    private static String folder = "triominoes/";


	
	
	/*
	 * constructor
	 */
	public ServerCommunication(Context mCtx){
		//mDbHelper = new dbAdapter(ServerCommunication.this);
		mDbHelper = new triDbAdapter(mCtx);
		
	}
	
	public  void close() {
		if (mDbHelper != null) {
     		mDbHelper.close();
		}
	}
	
	/* Methods
     *
     *
     */
	public static int getId() {
		return chPlayer2_ID;
	}
	public static long getGame() {
		return chGameID;
	}
	public static String getName() {
		return chPlayer2name;
	}
	
	void setName(String newValue) {
		chPlayer2name = newValue;
    }	
	void setGame(long newValue) {
		chGameID = newValue;
    }	
	void setId(int newValue) {
		chPlayer2_ID = newValue;
    }
	
	/*
	 * registerUser takes a preferred username and checks to see if it is unique
	 * among other users on the server
	 */
	public Boolean registerUser(String username) throws JSONException{ 
		JSONObject json = new JSONObject();
		String text;
		
		// JSON data:
		json.put("username", username);
		
		text = postPullData(json, server + folder + "/register.php");
		
		if (text.contains("Error")){
			return false;
		}
		/*
		 * expects a JSON object in the form of:
		 *  {"user": {"name": "some-name", "_id": 123} }
		 */
	    JSONObject jReturnObject = new JSONObject(text);
	    JSONObject nameObject = jReturnObject.getJSONObject("user");
	    String name = nameObject.getString("name");
	    int user_id = nameObject.getInt("_id");
	    mDbHelper.open();
	    long ret = mDbHelper.createUserProfile(name,user_id);
	    mDbHelper.close();
	    return (ret > 0);
		
	}

	/*
	 * gameChallenge takes the local player's username and user_id
	 * and challenges another user to play a game. The local user
	 * may specify a known username or may use the keyword
     * "_random_player" to select any old fool from the database
     * 
     * upon successful completion it populates the local dB with 
     *     - gameID, 9 tiles, player2's name and ID
     * returns -1 if error, or returns valid game_id if successful
     * an error is returned if the selected username doesn't exist
	 */
    public Long gameChallenge(String userName, int user_id, String OpponentName) throws JSONException {
    	JSONObject json = new JSONObject();
  		String text;
  		long game_id = -1;
  		
  	    // JSON data:
  		json.put("username", userName);
  		json.put("user_id", user_id);
  		json.put("username2", OpponentName);
  		
  		text = postPullData(json, server + folder + "/gamechallenge.php");
  		
  		if (text.contains("Error")){
			return (long) -1;
		}
  		/*
		 * expects a JSON object in the form of:
		 *  { "game_id": 11, "player2": "Rufus", "user2_id": 456
		 *  "tiles": [43,11,1,3,4,55,35,23,25] }
		*/
		JSONObject game = (JSONObject) new JSONTokener(text).nextValue();
		String player2name = game.getString("player2");
	    game_id = game.getLong("game_id");
	    int remote_id = game.getInt("user2_id");
	    JSONArray tiles = game.getJSONArray("tiles");
	    mDbHelper.open();
	    mDbHelper.createNewGame(game_id, user_id, remote_id, userName, player2name );
	    for (int i = 0; i < tiles.length(); i++) {
	    	int tile = tiles.getInt(i);
	    	mDbHelper.UpdateTiles(game_id, tile, 0, false);

	    }
	    mDbHelper.close();
	    return game_id;
    	
    }
    
    /*
     * sologame creates a game for one on the server. Need only the local player's
     * username and id
     */
    public long sologame(String userName, int user_id) throws JSONException {
    	JSONObject json = new JSONObject();
  		String text;
  		long game_id = -1;
  		
  	    // JSON data:
  		json.put("username", userName);
  		json.put("user_id", user_id);
  		
  		text = postPullData(json, server + folder + "/sologame.php");
  		
  		if (text.contains("Error")){
			return -1;
		}
  		
  		/*
		 * expects a JSON object in the form of:
		 *  { "game_id": 11, "player2": "Rufus", "user2_id": 456
		 *  "tiles": [43,11,1,3,4,55,35,23,25] }
		*/
		JSONObject game = (JSONObject) new JSONTokener(text).nextValue();
		String player2name = game.getString("player2");
	    game_id = game.getLong("game_id");
	    int remote_id = game.getInt("user2_id");
	    JSONArray tiles = game.getJSONArray("tiles");
	    mDbHelper.open();
	    mDbHelper.createNewGame(game_id, user_id, remote_id, userName, player2name );
	    for (int i = 0; i < tiles.length(); i++) {
	    	int tile = tiles.getInt(i);
	    	mDbHelper.UpdateTiles(game_id, tile, 0, false);

	    }
	    mDbHelper.close();
	    return game_id;
    }
    
     /*
      * findchallenges goes out to the server to see if there are
      * any new challenges given to this user_id.  If so, the local
      * Db is populated with the game information and the game is
      * accepted.  If notify is true then we won't create a game just yet
      * 
      */
     public Boolean findchallenges(String userName, int user_id, Boolean notify) throws JSONException {
    	 
    	JSONObject json = new JSONObject();
    	String text;
    	chPlayer2name = NO_CHALLENGE;
    	chPlayer2_ID = -1;
    	chGameID = -1;

    	// JSON data:
		json.put("user_id", user_id);
		
		text = postPullData(json, server + folder + "/findchallenges.php");
		
		if (text.contains("Error")){
			return false;
		}
		/*
		 * expects a JSON object in the form of:
		 *  { "game_id": 11, "player2": "Rufus", "user2_id": 456 }
		 */
		JSONObject game = (JSONObject) new JSONTokener(text).nextValue();
		
	    setName(game.getString("player2")); // set public vars
	    setGame(game.getLong("game_id"));
	    setId(game.getInt("user2_id"));
	    if (!notify) {     // only create games when this is not a notify event
		    mDbHelper.open();
		    mDbHelper.createNewGame(chGameID, user_id, chPlayer2_ID, userName, chPlayer2name );
		    checkplay(chGameID, user_id);
		    mDbHelper.close();
	    }
	    return true;
    	 
     }

     
     /**
      *      
       * playtile is called when the local user has played a tile and wants to pass the turn
      * to the opponent.  Required input are
      *   - game_id
      *   - local userid
      *   - tile, index of tiles played
      *   - fname, the enumerator of the tile, i.e. t334
      *   - row,col - where on the grid this tile played
      *   - score - user's updated score 
      * 
      * @throws JSONException
      */
     public void playTile(long game_id, int user_id, int who, int score, int tile, String fname,
             int row, int col, int rotate, Boolean facedn, int complete) throws JSONException {
    	 
    	 JSONObject json = new JSONObject();
   		 String text;
   		 
   	     // JSON data:
   		 json.put("game_id", game_id);
   		 json.put("user_id", user_id);
   		 json.put("score", score);
   		 json.put("tile", tile);
   		 json.put("fname",fname);
   		 json.put("row", row);
   		 json.put("col", col);
   		 json.put("rotate", rotate);
   		 int face = (facedn)? 1 :0;
   		 json.put("face_dn", face);
   		 json.put("who", who);
   		 json.put("complete", complete);
   		 
   		 text = postPullData(json, server + folder + "/playtile.php");
   		
    	 
     }


	/* 
	 * checkplay is called when the local user wants to check the other player's status.
	 * the current game state is compared to the online game state for changes in date/user.
	 * it updates the local database and passed on the pertinent info
	 * 
	 * assume database has been opened already by calling method. 
	 * 
	 */
     public Boolean checkplay(long game_id, int user_id) throws JSONException {
    	 JSONObject json = new JSONObject();
  		 String text;
  		 
  	     // JSON data:
  	     json.put("game_id", game_id);
  		 json.put("user_id", user_id);
  		
  		 text = postPullData(json,server + folder + "/checkplay.php" );
  		 if (text.contains("Error")){
  			 return false;
	     }
  		 /* expects a JSON object in the form of:
		  //$jdata = array("score"=>$score, "who"=>$who_id, "tile"=>$tile,
          //                "facenm"=>$fname, "row"=>$trow, "col"=>$tcol,
          //                "rotate"=>$trot, "facedn"=>$facedn, "date"=>$date, 
          //                "complete"=>complete);

		  
		 */
		 JSONObject jReturnObject = new JSONObject(text);
		 mDbHelper.open(); 
  	     int complete = jReturnObject.getInt("complete");
  	     if (complete == 1) {
	    	 mDbHelper.setGameOver(game_id);
	    	 mDbHelper.close(); 
	    	 return true;
	     }
  	     
         int new_who = jReturnObject.getInt("who");
          
		 if (mDbHelper.getUserId() == new_who) {  // local player played last tile, no news.
			if (mDbHelper != null) {
		         mDbHelper.close();
			}

   		    return false;
    	 }
          
         // fetch local Db lastplay game and compare the WHO key for last player.
         // that was local, then do nothing
         // the remote differs and we have a new play
         Cursor c = mDbHelper.fetchLastPlayed(game_id);
         if (c.moveToLast()) {
             int remote_id = c.getInt(c.getColumnIndex(triDbAdapter.KEY_WHO));
             int last_tile = c.getInt(c.getColumnIndex(mDbHelper.KEY_LASTT));     // this is the last tile the server knows of
             int tile = jReturnObject.getInt("tile");                             // this, however, is the last tile the local Db knows of
             if (new_who == remote_id) {
            	  if ((tile == last_tile)) {                    //   || (tile == -1)  || (last_tile == -1)){
            		 c.close();
            		 if (mDbHelper != null) {
            	    	mDbHelper.close();
            		 }
            		
            	     return false;                              // same as I already have, or user passed play
            	  }
             }	   
         }
         c.close();
         // if I've gotten here, then sound the alarms!
  	  	 // the remote user played a tile!
  		 // setup the rest of the JSON data and populate the local Db
  		 int oppscore = jReturnObject.getInt("score");
  		 int tile = jReturnObject.getInt("tile");
         String date = jReturnObject.getString("date");
  		 int row = jReturnObject.getInt("row");
  	     int col = jReturnObject.getInt("col");
  	     int rot = jReturnObject.getInt("rotate");
  	     String fname = jReturnObject.getString("facenm");
  	     int facednInt = jReturnObject.getInt("facedn");
  	     Boolean face_dn = (facednInt==1)?true:false;
  	     int remain = (mDbHelper.tilesRemaining(game_id))?1:0;
  	     
  	     

  	     mDbHelper.UpdateLastPlayed(game_id, mDbHelper.getUserId(), new_who, oppscore, tile, fname,
                    row, col, rot, face_dn, oppscore, remain, complete); // oppscore is repeated as it is turnScore in 2nd param
  	     mDbHelper.close();
  	     return true ;      
    	 
     }
     
     
  		
     
     
    /*
     *   passplay.php is called when a local user cannot play a tile and wishes to pass his turn
     *   - game_id and userid are required 
     *   Upon successful completion returns a new tile and updates the score on the remote server
     *   If no tiles were available, returns false;
     *   
     */
     public Boolean passplay(long game_id, int user_id) throws JSONException {
    	 JSONObject json = new JSONObject();
  		 String text;
  		 int tilesRemain = 1;
  		 
  	     // JSON data:
  		 json.put("game_id", game_id);
  		 json.put("user_id", user_id);
  		 
    	 text = postPullData(json, server + folder + "/passplay.php");
    	 mDbHelper.open();
    	 int old_score = mDbHelper.getScore(game_id, user_id);
    	 if (text.contains("Error")){
    		    tilesRemain = (text.contains("no tiles left")) ? 0 : 1;
    		    mDbHelper.UpdateLastPlayed(game_id, mDbHelper.getUserId(), user_id, old_score, 
    		    		 -1,"t---",0,0,0,false, 0, tilesRemain, 0);  // no tile, no score update, just tile update
    		    //mDbHelper.close();
				return false;
		 }
    	 /*
		 * expects a JSON object in the form of:
		 *  { "bone":  123, "score"; 456} }
		 */
		 JSONObject jReturnObject = new JSONObject(text);
		 int tilenum = jReturnObject.getInt("bone");
		 int score = jReturnObject.getInt("score");		// this score should be -5 if I got a tile back
		 int new_score = score + old_score;

		 mDbHelper.UpdateLastPlayed(game_id, mDbHelper.getUserId(), user_id, new_score, 
				 -1,"t---",0,0,0,true, -5, tilesRemain, 0);  // got a tile, so decrement score and update tile
		 Boolean ret = false;
		 if (user_id == 1) {  // this is the Android player
			 ret = mDbHelper.UpdateDeviceTiles(game_id, tilenum, new_score, true);
		 }
		 else {
		     ret = mDbHelper.UpdateTiles(game_id, tilenum, new_score, true);
		 }
		 mDbHelper.close();
		 return ret;
     }
     
     

     public String setGameOver(long game_id) {
    	 JSONObject json = new JSONObject();
  		 String text = null;
  		 
  	     // JSON data:
  		 try {
			json.put("game_id", game_id);
		} catch (JSONException e) {
			e.printStackTrace();
		}
    	try {
			text = postPullData(json, server + folder + "/gameover.php");
			
			if (text.contains("Error")){
				return text;
			}
		} catch (JSONException e) {
			e.printStackTrace();
			return e.toString();
		}
		return text;
    	 
     }
     
     /*
      * setFinalScore takes a game_id and the two scores and places them in the 
      * server game record. This is only used for when the player is playing against
      * the device. Totally not needed for normal game play, but without the 
      * game stats/leader board only shows high score for 2-person games. This will
      * then allow 1-person games against Android to begin to show up on the boards.
      */

     public void setFinalScore(long game_id, int p1score, int p2score) {
    	 JSONObject json = new JSONObject();
  		 String text;
  		 
  	     // JSON data:
  		 try {
			json.put("game_id", game_id);
			json.put("player1score", p1score);
			json.put("player2score", p2score);
		} catch (JSONException e) {
			e.printStackTrace();
		}
    	try {
			text = postPullData(json, server + folder + "/finalscore.php");
			
			if (text.contains("Error")){
				// tell someone
			}
		} catch (JSONException e) {
			e.printStackTrace();
		}
     }
    
    /*
     * gameAccept - this is called when a remote user has given a challenge to the
     * local user and the local user wishes to play a game. 
     * - inputs are 
     * The local user's dB is populated with 
     *  - local user's tiles 
     *  - opponent's username 
     *  - opponent's userid
     *  - game_id of the game
     *  - future - rows/columns of prizes/booby traps in created grid.
     */
    public void gameAccept(int remote_id, int local_id, long game_id, String userName, Boolean accepted) throws JSONException {
    	JSONObject json = new JSONObject();
  		String text;
  		
  	    // JSON data:
  		json.put("user1_id", remote_id);  //initiator
  		json.put("user2_id", local_id);   // acceptor
  		json.put("game_id",  game_id);    // game id
  		
  		text = postPullData(json, server + folder + "/gameaccept.php");
  		if (text.contains("Error")){
			return ;
		}
  		else {
  			/*
			 * expects a JSON object in the form of:
			 *  { "game_id": 11, "tiles": [43,11,1,3,4,55,35,23,25] }
			 */
			JSONObject game = (JSONObject) new JSONTokener(text).nextValue();
		    //int ret_game_id = game.getInt("game_id");
		    JSONArray tiles = game.getJSONArray("tiles");
		    //String player2name = getName();
		   
		    if (accepted) {
		    	mDbHelper.open();
			    for (int i = 0; i < tiles.length(); i++) {
			    	int tile = tiles.getInt(i);
			    	mDbHelper.UpdateTiles(game_id, tile, 0, false);
			    	

			    }
			    mDbHelper.close();
			    // check for any tiles played by opposing user
			    checkplay(game_id,local_id);
		    }
  		}
  		
    }

    public void gameAcceptDevice(int local_id, long game_id, String userName) throws JSONException {
    	JSONObject json = new JSONObject();
  		String text;
  		
  	    // JSON data:
  		json.put("user1_id", local_id);    // device is always user_id 1
  		json.put("user2_id", (int) 1);   // acceptor
  		json.put("game_id",  game_id);    // game id
  		
  		text = postPullData(json, server + folder + "/gameaccept.php");
  		if (text.contains("Error")){
			return ;
		}
  		else {
  			/*
			 * expects a JSON object in the form of:
			 *  { "game_id": 11, "tiles": [43,11,1,3,4,55,35,23,25] }
			 */
			JSONObject game = (JSONObject) new JSONTokener(text).nextValue();
		    JSONArray tiles = game.getJSONArray("tiles"); 
		    
		    // store the tiles in the Device's tile table
	    	mDbHelper.open();
		    for (int i = 0; i < tiles.length(); i++) {
		    	int tile = tiles.getInt(i);
		    	mDbHelper.UpdateDeviceTiles(game_id, tile, 0, false);
		    }
		    mDbHelper.close();
		    
  		}
  		
    }
    
    /*
     * pulls down all remaining tiles so device can act as local server
     * 
     */
    public void gameLocalServer(long game_id) {
    	JSONObject json = new JSONObject();
  		String text="";
  		int tile = -1;
  		JSONArray tiles = null;
  		
  	    // JSON data:
  		try {
			json.put("game_id",  game_id);
		} catch (JSONException e) {
			e.printStackTrace();
		}    // game id
  		
  		try {
			text = postPullData(json, server + folder + "/localserver.php");
		} catch (JSONException e) {
			e.printStackTrace();
		}
  		if (text.contains("Error")){
			return ;
		}
  		else {
  			/*
			 * expects a JSON object in the form of:
			 *  { "game_id": 11, "tiles": [43,11,1,3,4,55,35,23,25] }
			 */
			JSONObject game;
			try {
				game = (JSONObject) new JSONTokener(text).nextValue();
				tiles = game.getJSONArray("tiles"); 
			} catch (JSONException e) {
				e.printStackTrace();
			}
		    
		    
		    // store the tiles in the Device's tile table
	    	mDbHelper.open();
		    for (int i = 0; i < tiles.length(); i++) {
				try {
					tile = tiles.getInt(i);
				} catch (JSONException e) {
					e.printStackTrace();
				}
		    	mDbHelper.UpdateLocalServerTiles(game_id, tile);
		    }
		    mDbHelper.close();
  		}
    	
    }
    
    /*
     * the analog to gameAccept
     */
    public void deleteChallenge(long game_id, int user_id) {
    	JSONObject json = new JSONObject();
  		String text="";
  		
  	    // JSON data:
  		try {
			json.put("game_id", game_id);
			json.put("user_id", user_id);
		} catch (JSONException e) {
			e.printStackTrace();
		}    
		try {
			text = postPullData(json, server + folder + "/deletechallenge.php");
		} catch (JSONException e) {
			e.printStackTrace();
		}
		if (text.contains("Error")) {
			Log.d("SCOMM",text);
		}
  		
    }
    
    /*
     *  user has selected a "reset" and will delete all data
     *  this method communicates with server to set any games
     *  from this user as complete and will delete the name
     */
    public void deleteUser(int user_id) {
    	JSONObject json = new JSONObject();
  		String text="";
  		
  	    // JSON data:
  		try {
			json.put("user_id", user_id);
		} catch (JSONException e) {
			e.printStackTrace();
		}    
		try {
			text = postPullData(json, server + folder + "/deleteuser.php");
		} catch (JSONException e) {
			e.printStackTrace();
		}
		if (text.contains("Error")) {
			Log.d("SCOMM",text);
		}
    }

    /*
     * sendChat send a chat string to the server
     *  and expects no output. Since the user may want to
     *  see the chat after the fact, the chat is store locally
     *  in the database after having sent it to the server.
     */
    public void sendChat(long game_id, int user_id, String chat)throws JSONException {
    	JSONObject json = new JSONObject();
  		String text="";
  		
  	    // JSON data:
  		try {
			json.put("game_id", game_id);
			json.put("user_id", user_id);
			json.put("chat", chat);
		} catch (JSONException e) {
			e.printStackTrace();
		}    
		try {
			text = postPullData(json, server + folder + "/receivechat.php");  // I send, server receives
		} catch (JSONException e) {
			e.printStackTrace();
		}
		if (text.contains("Error")) {
			Log.d("SCOMM",text);
		}
		

    }

    /*
     * recvChat gets chat string(s) from the server
     *  each chat is stored locally
     *  in the database after having received it to the server.
     */
    public void recvChat(long game_id, int user_id) throws JSONException {
    	JSONObject json = new JSONObject();
  		String text="";
  		
  	    // JSON data:
  		try {
			json.put("game_id", game_id);
			json.put("user_id", user_id);
		} catch (JSONException e) {
			e.printStackTrace();
		}    
		try {
			text = postPullData(json, server + folder + "/sendchat.php");  // I receive, server sends
		} catch (JSONException e) {
			e.printStackTrace();
		}
		if (text.contains("Error")) {
			Log.d("SCOMM",text);
		}
		else {

			if (!text.equals(null)) {
				mDbHelper.open();
				JSONArray array = new JSONArray(text);
				for (int i = 0; i < array.length(); i++) {
				    JSONObject row = array.getJSONObject(i);
				    int who = row.getInt("who");
				    String said = row.getString("said");
				    mDbHelper.insertChat(game_id, who, said);
				}
				mDbHelper.close();
			}

		    
		}
		
    }

    /*
     * checkChat simply checks to see if there are any chats for this user, this game
     * @param - game_id - which game
     * @param - user_id - which user
     * 
     * return an integer of the number of unread messages on the server
     */
    public int checkChat(long game_id, int user_id) throws JSONException {
    	JSONObject json = new JSONObject();
  		String text="";
  		
  	    // JSON data:
  		try {
			json.put("game_id", game_id);
			json.put("user_id", user_id);
		} catch (JSONException e) {
			e.printStackTrace();
		}    
		try {
		text = postPullData(json, server + folder + "/checkchat.php");  // I receive, server sends
		} catch (JSONException e) {
			e.printStackTrace();
		}
		if (text.contains("Error")) {
			Log.d("SCOMM",text);
			return 0;
		}
		else {
			/*
			 * expects a JSON object in the form of:
			 *  { "msgcount":  123 }
			 */
			 JSONObject jReturnObject = new JSONObject(text);
			 int msgcount = jReturnObject.getInt("msgcount");
			 return msgcount;
		}
    }
    
   
   
    private String postPullData(JSONObject json, String URL) throws JSONException { 
    	HttpClient httpclient;
    	int CONNECTION_TIMEOUT = 20*1000;
    	int WAIT_RESPONSE_TIMEOUT = 20*1000;
		HttpPost httppost = new HttpPost(URL);
		
		String Error = "SCOMM Error: ";
		JSONArray postjson;
		URL url = null;
		             	
		int sdkVersion = Build.VERSION.SDK_INT;
		
        /*
         *  according to the Android docs, Android 3.0+ should use URLConnection
         *  but I can't get that to work. For now, keep using HttpConnection
         
		if (sdkVersion < Build.VERSION_CODES.HONEYCOMB) {
		*/
			// use the older HttpClient 
			
			HttpParams httpParameters = new BasicHttpParams();
			HttpConnectionParams.setConnectionTimeout(httpParameters, CONNECTION_TIMEOUT);
			HttpConnectionParams.setSoTimeout(httpParameters, WAIT_RESPONSE_TIMEOUT);
			HttpConnectionParams.setTcpNoDelay(httpParameters, true);

			httpclient = new DefaultHttpClient(httpParameters);
			
			
			try {
				
				postjson = new JSONArray();
				postjson.put(json);

				// Post the data:
				httppost.setHeader("json", json.toString());
				httppost.getParams().setParameter("jsonpost", postjson);

				// Execute HTTP Post Request
				System.out.print(json);
				HttpResponse response = httpclient.execute(httppost);

				// for JSON:
				if (response != null)
				{
					InputStream is = response.getEntity().getContent();
					BufferedReader reader = new BufferedReader(
							new InputStreamReader(is));
					StringBuilder sb = new StringBuilder();
					String line = null;
					try {
						while ((line = reader.readLine()) != null) {
						sb.append(line + "\n");
						}
					} catch (IOException e) {
						Error += " IO Exception: " + e;
						Log.d("SCOMM", Error);
						e.printStackTrace();
						return Error;
					} finally {
						try {
							is.close();
						} catch (IOException e) {

							Error += " closing connection: " + e;
							Log.d("SCOMM", Error);
							e.printStackTrace();
							return Error;
						}
					}
					
					return sb.toString();
				}
				else {
					Error += " no response from connection";
					Log.d("SCOMM", Error);
					return Error;
				}


			} catch (ClientProtocolException e) {

				Error += " client protocol err: " + e;
				Log.d("SCOMM", Error);
				e.printStackTrace();
				return Error;

			} catch (IOException e) {

				Error += " post IO exception " + e;
				Log.d("SCOMM", Error);
				e.printStackTrace();
				return Error;

			}

		/*
	     *  according to the Android docs, Android 3.0+ should use URLConnection
	     *  but I can't get that to work. For now, keep using HttpConnection
	         
			
			
		}
		else {
		
			// prefer the new URLConnection
        
			// setup the connection URL to the server
			try {
				url = new URL(URL);
			} catch (MalformedURLException e) {
				Error += "MalformedURLException  " + e;
				Log.d("SCOMM", Error);
				e.printStackTrace();
				return Error;
			}
					
			
			StringBuilder sb = new StringBuilder();
	
	
	        // open the connection
			try {
				
				URLConnection connection = (HttpURLConnection) url.openConnection();
				connection.setRequestProperty("Content-type","application/json"); 
				connection.setDoOutput(true);
				connection.setDoInput(true);
	
				connection.setUseCaches(false);
				connection.setAllowUserInteraction(false);
				((HttpURLConnection) connection).setRequestMethod("POST");  
				((HttpURLConnection) connection).setRequestMethod("PUT");
				
				// setup the write
				OutputStreamWriter out = new OutputStreamWriter(connection.getOutputStream());
				
				// write it
				out.write(json.toString());
				out.flush();
				
				// close the write stream
				//out.close();
				
				if (((HttpURLConnection) connection).getResponseCode() != 200 && ((HttpURLConnection) connection).getResponseCode() != HttpURLConnection.HTTP_NO_CONTENT) {
					throw new IOException(((HttpURLConnection) connection).getResponseMessage());
				}
				
				// setup the read stream
				BufferedReader in = new BufferedReader( new InputStreamReader( connection.getInputStream()));
				
				// read it
				String decodedString = null;
				while ((decodedString = in.readLine()) != null) {
					sb.append(decodedString + "\n");
				}
				
				// close the read stream
				in.close();
			} catch (IOException e) {
				Error += " post IO exception " + e;
				Log.d("SCOMM", Error);
				e.printStackTrace();
				return Error;
			}
	        
	
	        return sb.toString();

		}
		*
		*  don't use URLConnection
		**/

	}

    
    

}