package com.ulsanonline.triominoes;

/*
 * 02/22/2012 - v1.001 Initial release
 * 03/10/2012 - v2.0   added noplay2
 * 09/30/2012 - v3.4   new soundscheme and sounds.
 * 11/17/2012 - v3.5   pay attention to system settings for sounds
 * 
 */

import java.util.HashMap;
import java.util.Random;

import android.content.Context;
import android.media.AudioManager;
import android.media.SoundPool;

public class SoundManager {
	 
	static private SoundManager _instance;
	private static SoundPool mSoundPool;
	private static HashMap<Integer, Integer> mSoundPoolMap = new HashMap<Integer, Integer>();
	private static AudioManager  mAudioManager;
	private static Context mContext;
	
	public enum Sounds {none(0), droptile(1), noplay(2), pass(3), point(4), 
		                recall(5), rotate(6), game_winner(7), click(8), noplay2(9),
		                ok(10), wow(11), woohoo(12), point2(13), uhuh(14),
		                noway(15), sorry(16), dude(17), cant(18); 

		private final int sound;

		Sounds(int asound){
			this.sound = asound;
		}
		public int sound(){
			return this.sound;
		}
	}	
 
	private SoundManager()
	{
	}
 
	/**
	 * Requests the instance of the Sound Manager and creates it
	 * if it does not exist.
	 *
	 * @return Returns the single instance of the SoundManager
	 */
	static synchronized public SoundManager getInstance()
	{
	    if (_instance == null)
	      _instance = new SoundManager();
	    return _instance;
	 }
 
	/**
	 * Initialises the storage for the sounds
	 *
	 * @param theContext The Application context
	 */
	public static  void initSounds(Context theContext)
	{
		 mContext = theContext;
	     mSoundPool = new SoundPool(4, AudioManager.STREAM_MUSIC, 0);
	     mSoundPoolMap = new HashMap();
	     mAudioManager = (AudioManager)mContext.getSystemService(Context.AUDIO_SERVICE);
	     
	} 
 
	/**
	 * Add a new Sound to the SoundPool
	 *
	 * @param Index - The Sound Index for Retrieval
	 * @param SoundID - The Android ID for the Sound asset.
	 */
	public static void addSound(int Index,int SoundID)
	{
		mSoundPoolMap.put(Index, mSoundPool.load(mContext, SoundID, 1));
	}
 
	/**
	 * Loads the various sound assets
	 * Currently hardcoded but could easily be changed to be flexible.
	 */
	public static void loadSounds()
	{
		
		mSoundPoolMap.put(1, mSoundPool.load(mContext, R.raw.droptile, 1));
		mSoundPoolMap.put(2, mSoundPool.load(mContext, R.raw.noplay, 1));
		mSoundPoolMap.put(3, mSoundPool.load(mContext, R.raw.pass, 1));
		mSoundPoolMap.put(4, mSoundPool.load(mContext, R.raw.point, 1));
		mSoundPoolMap.put(5, mSoundPool.load(mContext, R.raw.recall, 1));
		mSoundPoolMap.put(6, mSoundPool.load(mContext, R.raw.rotate, 1));
		mSoundPoolMap.put(7, mSoundPool.load(mContext, R.raw.game_winner, 1));
		mSoundPoolMap.put(8, mSoundPool.load(mContext, R.raw.click, 1));
		mSoundPoolMap.put(9, mSoundPool.load(mContext, R.raw.noplay2, 1));
		mSoundPoolMap.put(10, mSoundPool.load(mContext, R.raw.ok, 1));
		mSoundPoolMap.put(11, mSoundPool.load(mContext, R.raw.wow, 1));
		mSoundPoolMap.put(12, mSoundPool.load(mContext, R.raw.woohoo, 1));
		mSoundPoolMap.put(13, mSoundPool.load(mContext, R.raw.point2, 1));

		mSoundPoolMap.put(14, mSoundPool.load(mContext, R.raw.uhuh, 1));
		mSoundPoolMap.put(15, mSoundPool.load(mContext, R.raw.noway, 1));
		mSoundPoolMap.put(16, mSoundPool.load(mContext, R.raw.sorry, 1));
		mSoundPoolMap.put(17, mSoundPool.load(mContext, R.raw.dude, 1));
		mSoundPoolMap.put(18, mSoundPool.load(mContext, R.raw.cant, 1));

	}
 
	/**
	 * Plays a Sound
	 *
	 * @param index - The Index of the Sound to be played
	 * @param speed - The Speed to play not, not currently used but included for compatibility
	 */
	public static void playSound(int index,float speed)
	{
		
		if (Settings.EnableSound) {
		     float streamVolume = mAudioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
		     streamVolume = streamVolume / mAudioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
		     mSoundPool.play(mSoundPoolMap.get(index), streamVolume, streamVolume, 1, 0, speed);
		}
	}
	
	public static void playRandomBadMoveSound(){
		Random r = new Random();
    	int range = Sounds.cant.sound() - Sounds.uhuh.sound() + 1;
    	int randomInt = Sounds.uhuh.sound() + r.nextInt(range);
		int speed = 1;
		playSound(randomInt,speed);  
		
	}
	
 
	/**
	 * Stop a Sound
	 * @param index - index of the sound to be stopped
	 */
	public static void stopSound(int index)
	{
		mSoundPool.stop(mSoundPoolMap.get(index));
	}
 
	/**
	 * Deallocates the resources and Instance of SoundManager
	 */
	public static void cleanup()
	{
		mSoundPool.release();
		mSoundPool = null;
	    mSoundPoolMap.clear();
	    mAudioManager.unloadSoundEffects();
	    _instance = null;
 
	}
 
}
