	package com.ulsanonline.triominoes;
	/*
	 * 02/25/2012 - Initial release

	 * 02/29/2012 - v1.003 - findChallenges may now be called during wakeup and alert
	 *                       so notification can either be about a game piece played or
	 *                       someone has made a challenge to this user.
	 * 03/26/2012 - v3.1   - don't check on solo games or games against Android
	 * 07/15/2012 - v3.2   - read game play status in sharedPreferences 
	 * 11/21/2012 - v3.5   - threading required for scomm methods
     * 12/22/2012 - v3.9   - game_id: int -> long
     * 01/30/2013 - v4.1   - ensure getSettings called on startup/init
	 *                      
	 */
	import org.json.JSONException;
	
	import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.os.StrictMode;
import android.util.Log;

import com.ulsanonline.triominoes.Settings;
	
	
	public class WakeCheck extends Service {
	
		private triDbAdapter mDbHelper;
		private ServerCommunication sComm;
        private static final int CHALLENGE = -2;

        final static private String ACCOUNT_PREFS_NAME = "prefs";
        final static private String ACCESS_REMOTE = "ACCESS_REMOTE_KEY";
	
	@Override
	public void onCreate() {

	}
	
	
	
	@Override
	public IBinder onBind(Intent intent) {
	return null;
	}
	
	
	
	@Override
	public void onDestroy() {
	super.onDestroy();
	mDbHelper.close();

	}
	
	
	
	@Override
	public void onStart(Intent intent, int startId) {
		super.onStart(intent, startId);
		
		// get sharedPreferences setting for checking remote plays
		SharedPreferences prefs = getSharedPreferences(ACCOUNT_PREFS_NAME, 0);
		Boolean accessRemotePlays = prefs.getBoolean(ACCESS_REMOTE, false);
		
		if (accessRemotePlays) {
			
			Cursor c;
			mDbHelper = new triDbAdapter(this.getApplicationContext());
			
			sComm = new ServerCommunication(this.getApplicationContext());
			
			// did remote player make a move, or any new challenges and new games?
			checkRemotePlays();
			
			sComm.close();
		}
	}
	
	 private void checkRemotePlays() {	
	    	     	
    	new AsyncTask<Void,Void, Boolean>(){
            @Override
            protected void onPostExecute(Boolean isFound) {
            	Log.d("WAKE", "CheckRemotePlays has finished"+isFound);
            }

			@Override
			protected Boolean doInBackground(Void... params) {
				boolean playMade = false;
				mDbHelper.open();
				Cursor c = mDbHelper.fetchAllGames();
				
				if (c.getCount() > 0){
					c.moveToFirst();
					do {
						long game_id = c.getInt(c.getColumnIndex(mDbHelper.KEY_ID));
						if ( (!mDbHelper.isSoloGame(game_id)) && 
							 (!mDbHelper.getOpponentName(game_id).contains("ANDROID")) ) {
							// this is not a solo game and the user is not playing his own phone/tablet, then check 
							
							playMade = false;
							try {
			    	        	 playMade = sComm.checkplay(game_id, 0);
										
							}
			    	        catch (JSONException e) {
								 e.printStackTrace();
								 Log.d("WAKE", "Something went wrong in JSON");
						
						    }
							
							if (playMade) {
								//notify user - communicate that there has been a new play
				      		    //SoundManager.playSound(6, 1);
		
								String name = c.getString(c.getColumnIndex(mDbHelper.KEY_OPPNM));
								int tscore = c.getInt(c.getColumnIndex(mDbHelper.KEY_TSCORE));
								setUpNotification(name,tscore);
						    } 

						}
					} while (c.moveToNext());
				}
				c.close();
				mDbHelper.close();
				
				// check for challenges to local user
				mDbHelper.open();
		    	final String userName = mDbHelper.getName();
				final int user_id = mDbHelper.getUserId();
				boolean challengeMade = false;
				try {
					challengeMade =  sComm.findchallenges(userName, user_id, true);
				}
			
				catch (JSONException e) {
					e.printStackTrace();
				}
				catch (ClassCastException e) {
		    		Log.d("WAKE","Communication Error");
					e.printStackTrace();
				}
				if (challengeMade) {
					setUpNotification(sComm.chPlayer2name, CHALLENGE);
				}
				
				mDbHelper.close();
				
				return playMade;
    	        
			}

        }.execute();

	    	
	}
	    
	
	private void setUpNotification(String name, int tscore) {
		String ns = this.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) getSystemService(ns);
		int icon = R.drawable.status_icon;
		CharSequence tickerText = "TriOminoes!";
		long when = System.currentTimeMillis();

		Notification notification = new Notification(icon, tickerText, when);
		Context context = getApplicationContext();
		CharSequence contentTitle = "TriOminoes";
		CharSequence contentText = "";
		if (tscore == CHALLENGE) {
			// this is a challenge notification
		    contentText = name + " challenged you to a game!";	
		}
		else {
		    contentText = name + " played for " + tscore +
		                " points. It's Your Turn!";
		}
		Intent notificationIntent = new Intent(this, SelectGames.class);
		
		// when the user selects this notification, there's no need to
		// do another server check from SelectGames
		Bundle bundle = new Bundle();
    	bundle.putInt("NO_CHECK", 1);
		notificationIntent.putExtras(bundle); 
		
		// get settings and notify user as per his or her settings values
		mDbHelper.getSettings();
		
		if (Settings.AlarmVibrate) {
			notification.defaults |= Notification.DEFAULT_VIBRATE;
		}
		if (Settings.AlarmSound) {
			notification.defaults |= Notification.DEFAULT_SOUND;
		}
		notification.defaults |= Notification.FLAG_AUTO_CANCEL;
		

		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, notificationIntent,
				PendingIntent.FLAG_CANCEL_CURRENT );
		
		notification.setLatestEventInfo(context, contentTitle, contentText, contentIntent);
		
		

		mNotificationManager.notify(1, notification);

	}
	
	@Override
	public boolean onUnbind(Intent intent) {
		mDbHelper.close();
		return super.onUnbind(intent);
	}
	
	
	
	}
