	package com.ulsanonline.triominoes;
	/*
	 * 02/22/2012 - v1.001 Initial release
	 * 03/01/2012 - v1.004 add numremain to games table to hold count of remaining tiles
	 *                     added getNumRemaining and setNumRemaining
	 *                     added fetchAllGames variants to differentiate myturn from theirs
	 * 03/10/2012 - v2.0   allow user to play solo games
	 * 03/11/2012 - v3.0   added in-game chat.  chats table, methods to insert/fetch chats
	 * 03/18/2012 - v3.1 - play against the Android computer
	 * 11/22/2012 - v3.5 - added default values, check for no data. fetchAllGames - only incomplete
	 * 12/04/2012 - v3.8 - initialize user profile with new flag
	 * 12/22/2012 - v3.9 - game_id: int -> long
	 * 01/06/2013 - v4.0 - removed ShowCenters flag, added ShowAndroidTiles flag (reused table value) and 
	 *                     added AndroidPlayDifficult flag set/get
	 * 01/30/2013 - v4.1 - implement SQLite DB as singleton
	 * 09/29/2013 - v4.3 - added tile rotation to last played query for selectGames
	 * 
	 */
	
	import java.util.Calendar;
	
	import android.content.ContentValues;
	import android.content.Context;
	import android.database.Cursor;
	import android.database.SQLException;
	import android.database.sqlite.SQLiteDatabase;
	import android.database.sqlite.SQLiteOpenHelper;
    import android.util.Log;
	
	public class triDbAdapter {
	
	
	    
	    /**
	     * Database creation sql statement
	     */
	    private static final String DATABASE_CREATE1 =    
	    "CREATE TABLE player (" +
	    		  "_id              INTEGER ," +
	    		  "username         TEXT not null," +
	    		  "alarmtime        INTEGER not null," +
	    		  "alarmvibrate     INTEGER not null," +
	    		  "alarmsound       INTEGER not null," +
	    		  "style            INTEGER not null," +
	    		  "showboard        INTEGER not null," +
	    		  "showtouch        INTEGER not null," +
	    		  "showcenters      INTEGER not null," +
	    		  "showscores       INTEGER not null," +
	    		  "enablesound      INTEGER not null," +
	    		  "difficulty       INTEGER not null);" ;
	    
	    private static final String DATABASE_CREATE2 = 
	    "CREATE TABLE games (" +
	              "_id              INTEGER, " +
	              "myScore          INTEGER, " +
	              "username         TEXT,    " +
	              "myId             INTEGER, " +
	              "opponentId       INTEGER, " +
	              "opponentName     TEXT,    " +
	              "opponentScore    INTEGER, " +
	              "last_tile        INTEGER, " +
	              "last_tile_name   TEXT,    " +
	              "row              INTEGER, " +
	              "col              INTEGER, " +    // grid location & rotation, face tells
	              "rotate           INTEGER, " +    // all scoring information
	              "face_dn          INTEGER, " +    // 1=face down, 1=face pointing up
	              "who              INTEGER, " +    // who played it, user id
	              "tscore           INTEGER, " +    // just score per that turn
	              "remain           INTEGER, " +    // tiles remain(1) or not (0)
	              "numremain        INTEGER, " +    // how many remain
	              "complete         INTEGER, " +    // game over(1) or keep playing(0)
	              "last_played      INTEGER); " ;
	    
	    private static final String DATABASE_CREATE3 = 
	    "CREATE TABLE tilesPlayed (" +
	             "_id               INTEGER REFERENCES games(_id) on UPDATE CASCADE," +
	             "row               INTEGER, " +
	             "col               INTEGER, " +    // grid location & rotation, face tells
	             "rotate            INTEGER, " +    // all scoring information
	             "face_dn           INTEGER, " +    // 1=face down, 0=face pointing up
	             "tile              INTEGER); " ;
	    
	    private static final String DATABASE_CREATE4 = 
	    "CREATE TABLE myTiles  (" +
	             "_id               INTEGER PRIMARY KEY AUTOINCREMENT, " +
	             "game_id           INTEGER REFERENCES games(_id) on UPDATE CASCADE," +
	             "tile              INTEGER); " ;
	    
	    private static final String DATABASE_CREATE5 = 
		"CREATE TABLE gamechats  (" +
		        "_id               INTEGER PRIMARY KEY AUTOINCREMENT, " +
		        "game_id           INTEGER REFERENCES games(_id) on UPDATE CASCADE," +
		        "who               INTEGER, " +
		        "date              INTEGER, " +
		        "chat              TEXT); " ;
	    
	    private static final String DATABASE_CREATE6 = 
		"CREATE TABLE deviceTiles  (" +
		        "_id               INTEGER PRIMARY KEY AUTOINCREMENT, " +
		        "game_id           INTEGER REFERENCES games(_id) on UPDATE CASCADE," +
		        "tile              INTEGER); " ;
	    
	    private static final String DATABASE_CREATE7 = 
		"CREATE TABLE localServerTiles  (" +
			    "_id               INTEGER PRIMARY KEY AUTOINCREMENT, " +
			    "game_id           INTEGER REFERENCES games(_id) on UPDATE CASCADE," +
			    "tile              INTEGER); " ;
	              
	    private static final String DATABASE_NAME = "triominoes.db";
	    private static final String PLAYER_TABLE  = "player";
	    private static final String GAMES_TABLE   = "games";
	    private static final String TILES_TABLE   = "tilesPlayed";
	    private static final String MYTILES_TABLE = "myTiles";
	    private static final String CHATS_TABLE   = "gamechats";
	    private static final String DEVTILE_TABLE = "deviceTiles";
	    private static final String LSERVER_TABLE = "localServerTiles";
	    private static final int DATABASE_VERSION = 7;  
	              
	    public static final String KEY_NAME   = "username";
	    public static final String KEY_ID     = "_id";
	    public static final String KEY_GAME   = "game_id";
	    public static final String KEY_USERID = "myId";
	    public static final String KEY_TILE   = "tile";
	    public static final String KEY_LASTT  = "last_tile"; 
	    public static final String KEY_LASTN  = "last_tile_name";
	    public static final String KEY_SCORE  = "myScore";
	    public static final String KEY_OPP    = "opponentId";
	    public static final String KEY_OPPSC  = "opponentScore";
	    public static final String KEY_OPPNM  = "opponentName";
	    public static final String KEY_DATE   = "last_played";
	    public static final String KEY_ROW    = "row";
	    public static final String KEY_COL    = "col";
	    public static final String KEY_ROT    = "rotate";
	    public static final String KEY_FACEDN = "face_dn";
	    public static final String KEY_TSCORE = "tscore";
	    public static final String KEY_WHO    = "who";
	    public static final String KEY_REMAIN = "remain";
	    public static final String KEY_NUMREMAIN = "numremain";
	    public static final String KEY_COMPLETE = "complete";
	    public static final String KEY_STYLE  = "style";
	    public static final String KEY_ATIME  = "alarmtime";
	    public static final String KEY_AVIBE  = "alarmvibrate";
	    public static final String KEY_ASOUND = "alarmsound";
	    public static final String KEY_SHOWBOARD = "showboard";
	    public static final String KEY_SHOWTOUCH = "showtouch";
	    public static final String KEY_SHOWCENTERS = "showcenters";
	    public static final String KEY_SHOWSCORE = "showscores";
	    public static final String KEY_SOUND = "enablesound";
	    public static final String KEY_DIFFICULTY = "difficulty";
	    public static final String KEY_CHAT = "chat";
	    public static final String KEY_WHEN = "date";
	    
	    private static final String ALTER_TABLE1 = 
	    	"ALTER TABLE player ADD COLUMN enablesound INTEGER DEFAULT 1";
	    private static final String ALTER_TABLE2 = 
		    "ALTER TABLE games ADD COLUMN numremain INTEGER DEFAULT 38";
	    private static final String ALTER_TABLE3 = 
	    	"ALTER TABLE player ADD COLUMN showscores INTEGER default 1";
	    private static final String ALTER_TABLE4 = 
	    	"ALTER TABLE player ADD COLUMN difficulty INTEGER default 0";
	    
	    private static final String TAG = "dbAdapter";
	    private DatabaseHelper mDbHelper;
	    private SQLiteDatabase mDb;
	    private final Context mCtx;
		private static final String DATE_FORMAT_NOW = "yyyy-MM-dd hh:mm";
	
	
	    public static class DatabaseHelper extends SQLiteOpenHelper {
	    	
	    	private static DatabaseHelper mInstance = null;
	    	
	    	public static DatabaseHelper getInstance(Context ctx) {
	    	      
	    	    // Use the application context, which will ensure that you
	    	    // don't accidentally leak an Activity's context.
	    	    // See this article for more information: http://bit.ly/6LRzfx
	    	    if (mInstance == null) {
	    	      mInstance = new DatabaseHelper(ctx.getApplicationContext());
	    	    }
	    	    return mInstance;
	    	}
	    	     
	    	/**
	    	 * Constructor should be private to prevent direct instantiation.
	    	 * make call to static factory method "getInstance()" instead.
	    	*/
	    	private DatabaseHelper(Context ctx) {
	    	    super(ctx, DATABASE_NAME, null, DATABASE_VERSION);
	    	}
	

	
	        
	        
	        @Override
	        public void onCreate(SQLiteDatabase db) {
	
	            db.execSQL(DATABASE_CREATE1);
	            db.execSQL(DATABASE_CREATE2);
	            db.execSQL(DATABASE_CREATE3);
	            db.execSQL(DATABASE_CREATE4);
	            db.execSQL(DATABASE_CREATE5);
	            db.execSQL(DATABASE_CREATE6);
	            db.execSQL(DATABASE_CREATE7);
	
	
	        }
	        
	        @Override
	        public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
	            Log.w(TAG, "Upgrading database from version " + oldVersion + " to "
	                    + newVersion + ". No data will be destroyed");
	            if (oldVersion == 1) {
	            	db.execSQL(ALTER_TABLE1);
	            	db.execSQL(ALTER_TABLE2);
	            	db.execSQL(DATABASE_CREATE5);
	            	db.execSQL(DATABASE_CREATE6);
		            db.execSQL(DATABASE_CREATE7);
		            db.execSQL(ALTER_TABLE3);
		            db.execSQL(ALTER_TABLE4);

	            }
	            if (oldVersion == 2) {
	            	db.execSQL(ALTER_TABLE2);
	            	db.execSQL(DATABASE_CREATE5);
	            	db.execSQL(DATABASE_CREATE6);
		            db.execSQL(DATABASE_CREATE7);
		            db.execSQL(ALTER_TABLE3);
		            db.execSQL(ALTER_TABLE4);

	            }
	            if (oldVersion == 3) {
	            	db.execSQL(DATABASE_CREATE5);
	            	db.execSQL(DATABASE_CREATE6);
		            db.execSQL(DATABASE_CREATE7);
		            db.execSQL(ALTER_TABLE3);
		            db.execSQL(ALTER_TABLE4);

	            }
	            if (oldVersion == 4) {
	            	db.execSQL(DATABASE_CREATE6);
		            db.execSQL(DATABASE_CREATE7);
		            db.execSQL(ALTER_TABLE3);
		            db.execSQL(ALTER_TABLE4);

	            }
	            if (oldVersion == 5) {
		            db.execSQL(ALTER_TABLE3);
		            db.execSQL(ALTER_TABLE4);
	            }
	            
	            if (oldVersion == 6) {
	            	db.execSQL(ALTER_TABLE4);
	            }
	
	        }
	        
	    }    
	        
	    /**
	     * Constructor - takes the context to allow the database to be
	     * opened/created
	     * 
	     * @param ctx the Context within which to work
	     */
	    public  triDbAdapter(Context ctx) {
	        this.mCtx = ctx;
	    }
	    
	    /**
	     * Open the student database. If it cannot be opened, try to create a new
	     * instance of the database. If it cannot be created, throw an exception to
	     * signal the failure
	     * 
	     * @return this (self reference, allowing this to be chained in an
	     *         initialization call)
	     * @throws SQLException if the database could be neither opened or created
	     */
	    public triDbAdapter open() throws SQLException {
	        mDbHelper = DatabaseHelper.getInstance(mCtx);
	        mDb = mDbHelper.getWritableDatabase();
	        return this;
	    }
	
	    public void close() {
	    	//if (mDbHelper != null) {
	     	//	mDbHelper.close();
			//}
			//else {
			//	Log.d("DB:", "why was my db null?");
			//}
	    }
	
	    /**
	     * createUserProfile inserts the local username and server-generated userid, _ID , into the database.
	     * @param username the name of the local user
	     * @param _id - index in player table
	     * @return 
	     */    
	    public long createUserProfile(String username, int _id) {
	        ContentValues initialValues = new ContentValues();
	        initialValues.put(KEY_NAME, username);
	        initialValues.put(KEY_ID, _id);
	        initialValues.put(KEY_ATIME,Settings.AlarmInterval);
	        initialValues.put(KEY_AVIBE,Settings.AlarmVibrate?1:0);
	        initialValues.put(KEY_ASOUND,Settings.AlarmSound?1:0);
	        initialValues.put(KEY_STYLE, 0); // placeholder for future tile style
	        initialValues.put(KEY_SHOWBOARD,Settings.ShowBoard?1:0);
	        initialValues.put(KEY_SHOWTOUCH, Settings.ShowTouch?1:0);
	        initialValues.put(KEY_SHOWCENTERS,Settings.ShowAndroidTiles?1:0);
	        initialValues.put(KEY_SHOWSCORE, Settings.ShowScorePopUps?1:0);
	        initialValues.put(KEY_SOUND, Settings.EnableSound?1:0);
	        initialValues.put(KEY_DIFFICULTY, Settings.AndroidPlayDifficult?1:0);
	        return mDb.insertOrThrow(PLAYER_TABLE, null, initialValues);
	    }
	    
	    /* initially upon startup, get what's in the dB */
	    public void getSettings()  {
	    	Log.w(TAG, "getting settings");
	    	String sql = "SELECT * FROM player";
	    	Cursor c = mDb.rawQuery(sql, null);
	    	c.moveToLast();
	    	Settings.AlarmInterval = c.getInt(c.getColumnIndex(KEY_ATIME));
	    	Settings.AlarmVibrate = (c.getInt(c.getColumnIndex(KEY_AVIBE)) == 1);
	    	Settings.AlarmSound = (c.getInt(c.getColumnIndex(KEY_ASOUND)) == 1);
	    	Settings.ShowBoard = (c.getInt(c.getColumnIndex(KEY_SHOWBOARD)) == 1);
	    	Settings.ShowTouch = (c.getInt(c.getColumnIndex(KEY_SHOWTOUCH)) == 1);
	    	Settings.ShowAndroidTiles = (c.getInt(c.getColumnIndex(KEY_SHOWCENTERS)) == 1);
	    	Settings.ShowScorePopUps = (c.getInt(c.getColumnIndex(KEY_SHOWSCORE)) == 1);
	    	Settings.EnableSound = (c.getInt(c.getColumnIndex(KEY_SOUND)) == 1);
	    	Settings.AndroidPlayDifficult = (c.getInt(c.getColumnIndex(KEY_DIFFICULTY)) == 1);
	    	c.close();
	    }
	    
	    public void setAlarmInterval() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_ATIME, Settings.AlarmInterval);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    
	    public void setAlarmVibrate() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_AVIBE, Settings.AlarmVibrate);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    
	    public void setAlarmSound() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_ASOUND, Settings.AlarmSound);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	
	    public void setShowBoard() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_SHOWBOARD, Settings.ShowBoard);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    public void setShowCenters() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_SHOWCENTERS, Settings.ShowAndroidTiles);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    public void setShowTouch() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_SHOWTOUCH, Settings.ShowTouch);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    
	    public void setShowScorePopUps() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_SHOWSCORE, Settings.ShowScorePopUps);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    
	    public void setSound() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_SOUND, Settings.EnableSound);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    
	    public void setDifficulty() {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_DIFFICULTY, Settings.AndroidPlayDifficult);
	    	int ret = mDb.update(PLAYER_TABLE, args, null, null);
	    }
	    
	    /* if user makes changes to settings, stuff the db with the new values */
	    public void setSettings() {
	    	Log.w(TAG, "setting new settings");
	    }
	    public String getName() {
	    	
	    	Cursor c = mDb.query(PLAYER_TABLE, new String[] {KEY_ID, KEY_NAME}, null, null, null, null, null);
	    	
	    	//String sql = "SELECT * FROM player";
	    	//Cursor c =  mDb.rawQuery(sql, null); 
	        
	        String name =  "no_name";
	        if (c.moveToLast()) {
	            name = c.getString(c.getColumnIndex(KEY_NAME));
	        }
	        c.close();
	        return name;
	        
	    }
	    
	    /* 
	     * given a game_id, it returns a string with the name of the opponent.
	     */
	    public String getOpponentName(long game_id) {
	    	String sql = "SELECT * FROM games WHERE _id="+game_id;
	    	Cursor c =  mDb.rawQuery(sql, null); 
	        c.moveToLast();
	        String name =  c.getString(c.getColumnIndex(KEY_OPPNM));
	        c.close();
	        return name;
	    }
	    
	    public int getUserId() {
	    	String sql = "SELECT * FROM player";
	    	Cursor c =  mDb.rawQuery(sql, null); 
	    	int id = -1;
	        if (c.moveToLast()){
	           id = c.getInt(c.getColumnIndex(KEY_ID));
	        }
	        c.close();
	        return id;
	    	
	    }
	    
	    public Boolean tilesRemaining (long game_id) {
	    	String sql = "SELECT remain FROM games WHERE _id=" + game_id;
	    	Cursor c = mDb.rawQuery(sql, null);
	    	c.moveToLast();
	        int remain = c.getInt(c.getColumnIndex(KEY_REMAIN));
	        c.close();
	    	return (remain==1);
	    }
	    
	    public int getNumRemaining (long game_id) {
	    	String sql = "SELECT numremain FROM games WHERE _id=" + game_id;
	    	Cursor c = mDb.rawQuery(sql, null);
	    	c.moveToLast();
	        int remain = c.getInt(c.getColumnIndex(KEY_NUMREMAIN));
	        c.close();
	    	return (remain);
	    }
	    
	    public void setNumRemaining (long game_id, int count) {
	    	ContentValues args = new ContentValues();
	    	args.put(KEY_NUMREMAIN, count);
	    	mDb.update(GAMES_TABLE, args, KEY_ID + "=" + game_id, null);
	    }
	    
	    public Boolean gameOver (long game_id) {
	    	String sql = "SELECT complete FROM games WHERE _id=" + game_id;
	    	Cursor c = mDb.rawQuery(sql, null);
	    	c.moveToLast();
	        int complete = c.getInt(c.getColumnIndex(KEY_COMPLETE));
	        c.close();
	    	return (complete==1);
	    }
	    
	    public Boolean isItMyTurn(long game_id, int user_id) {
	    	if (isSoloGame(game_id)) {
	    		return true;
	    	}
	    	else {
		    	Cursor c = fetchLastPlayed(game_id);
		        if (c.moveToLast()) {
		            int last_player_id = c.getInt(c.getColumnIndex(KEY_WHO));
		            c.close();
		            if (last_player_id == user_id)
		            	return false;
		            else 
		            	return true;
		        }
		        c.close();
		        // if I got here, there aren't any records. Must be my turn
		        return true;
	    	}
	    }
	    
	    public int lastPlayed(long game_id) {
	    	String sql;
	    	String column;
	    	sql = "SELECT last_tile FROM games WHERE _id="+ game_id;
	    	Cursor c =  mDb.rawQuery(sql, null); 
	        c.moveToLast();
	        int tile =  c.getInt(c.getColumnIndex(KEY_LASTT));
	        c.close();
	        return tile;
	    }
	    
	    public String lastPlayedName(long game_id) {
	    	String sql;
	    	String column;
	    	sql = "SELECT last_tile_name FROM games WHERE _id="+ game_id;
	    	Cursor c =  mDb.rawQuery(sql, null); 
	        c.moveToLast();
	        String tname = c.getString(c.getColumnIndex(KEY_LASTN));
	        c.close();
	        return tname;
	    }
	    
	    public int getScore(long game_id, int who) {
	    	
	    	String sql;
	    	String column;
	    	if (who == getUserId()) {
	    		// this is my score
	    		sql = "SELECT myScore FROM games WHERE _id="+ game_id;
	    		column = KEY_SCORE;
	    	}
	    	else {
	    		// this is opponent's score
	    		sql = "SELECT opponentScore FROM games WHERE _id="+ game_id;
	    		column = KEY_OPPSC;
	
	    	}
	    	
	    	Cursor c =  mDb.rawQuery(sql, null); 
	        
	        int score = 0;
	        if (c.moveToLast()) {
	           score = c.getInt(c.getColumnIndex(column));
	        }
	        c.close();
	        return score;
	    	
	    }
	    
	    /**
	     * 
	     */
	    public Boolean isSoloGame(long game_id) {
	    	String sql = "SELECT opponentId FROM games WHERE _id=" + game_id;
	    	Cursor c = mDb.rawQuery(sql, null);
	    	c.moveToLast();
	        int opponent = c.getInt(c.getColumnIndex(KEY_OPP));
	        c.close();
	    	return (opponent == -1);  // -1 = NO opponent
	    }
	    
	    public Boolean isPlayingDevice(long game_id) {
	    	String sql = "SELECT opponentId FROM games WHERE _id=" + game_id;
	    	Cursor c = mDb.rawQuery(sql, null);
	    	
	        int opponent = 0;
	        if (c.moveToLast()) {
	        	opponent = c.getInt(c.getColumnIndex(KEY_OPP));
	        }
	        c.close();
	    	return (opponent == 1);   // 1 = the Android device
	    }
	    
	    
	    /*
	     * insertChat - inserts a chat string into the chats table
	     * @param - game_id which game
	     * @param - user_id - who said it
	     * @param - chat - what did he say
	     */
	    public void insertChat(long game_id, int user_id, String chat) {
	    	ContentValues initialValues = new ContentValues();
	        initialValues.put(KEY_GAME, game_id);
	        initialValues.put(KEY_WHO, user_id);
	        initialValues.put(KEY_CHAT, chat);
	        Calendar cal = Calendar.getInstance();
	        initialValues.put(KEY_WHEN,cal.getTimeInMillis());
	        mDb.insertOrThrow(CHATS_TABLE, null, initialValues);
	    }
	    
	    /*
	     * fetchAllChats - fetches all chats associated with
	     * @param game_id - which game 
	     *   and creates a cursor of game, who said it and what was said
	     */
	    public Cursor fetchAllChats(long game_id) {
	    	return mDb.query(CHATS_TABLE, new String[] {KEY_ID, KEY_GAME, KEY_WHO, KEY_CHAT, KEY_WHEN}, 
	                KEY_GAME+"="+game_id, null, null, null, KEY_WHEN);
	    }
	    
	    /**
	     * Return a Cursor over the list of all games the player has in the database
	     * @param - none
	     * @return Cursor over all games
	     */
	    public Cursor fetchAllGames() {
	
	        return mDb.query(GAMES_TABLE, new String[] {KEY_ID, KEY_SCORE,
	                KEY_OPP, KEY_OPPSC, KEY_OPPNM, KEY_DATE, KEY_LASTT, KEY_LASTN, KEY_TSCORE, KEY_WHO, KEY_REMAIN, KEY_COMPLETE}, 
	                KEY_COMPLETE+"<>1", null, null, null, KEY_DATE);
	    }
	    
	    /*
	     * fetches just those games in play, with the operator being either = or <> for who played it
	     */
	    public Cursor fetchAllGamesInPlay(int user_id, String operator) {
	    	return mDb.query(GAMES_TABLE, new String[] {KEY_ID, KEY_SCORE,
	                KEY_OPP, KEY_OPPSC, KEY_OPPNM, KEY_DATE, KEY_LASTT, KEY_LASTN, KEY_TSCORE, KEY_WHO, KEY_REMAIN, KEY_COMPLETE, KEY_ROT}, 
	                KEY_WHO+operator+user_id + " OR " + KEY_OPP+"=-1",
	                null, null, null, KEY_DATE);
	    }
	    
	    public Cursor fetchAllGamesDone() {
	    	return mDb.query(GAMES_TABLE, new String[] {KEY_ID, KEY_SCORE,
	                KEY_OPP, KEY_OPPSC, KEY_OPPNM, KEY_DATE, KEY_LASTT, KEY_LASTN, KEY_TSCORE, KEY_WHO, KEY_REMAIN, KEY_COMPLETE, KEY_ROT}, 
	                KEY_COMPLETE+"=1", null, null, null, KEY_DATE);
	    }
	    
	    /**
	     * Return a Cursor of a single game the player has in the database
	     * @param - none
	     * @return Cursor over a games
	     */
	    public Cursor fetchGame(long game_id) {
	    	Cursor c;
	 
	    	String sql = "SELECT _id,myScore, myId, username, opponentId, opponentName,OpponentScore,who  " +
	    			"FROM games WHERE _id=" + game_id;
	    	c =  mDb.rawQuery(sql, null); 
	        return c;	
	       
	    }
	    
	    /**
	     * returns a cursor over the latest player name and _id
	     * @param - none
	     * @return Cursor 
	     */
	    public Cursor fetchPlayer () {
	        return mDb.query(PLAYER_TABLE, new String[] {KEY_ID, KEY_NAME}, 
	        		null, null, null, null, KEY_NAME);
	 
	    }
	    /**
	     * fetch myTiles - returns a cursor over all the tiles in the local user's shelf
	     * @param game_id - which game the user is playing
	     */
	    public Cursor fetchPlayedTiles(long game_id){
	        Cursor c;
	    	
	    	String sql = "SELECT * FROM tilesPlayed " + 
	    	             "WHERE _id=" + game_id;
	
	        c =  mDb.rawQuery(sql, null); 
	        return c;	
	    }
	    
	    /**
	     * fetch myTiles - returns a cursor over all the tiles in the local user's shelf
	     * @param game_id - which game the user is playing
	     */
	    public Cursor fetchMyTiles(long game_id){
	        Cursor c;
	    	
	    	String sql = "SELECT tile FROM myTiles " + 
	    	             "WHERE game_id=" + game_id;
	
	        c =  mDb.rawQuery(sql, null); 
	        return c;	
	    }


        /* same as fetchtiles, but uses device table
         * 
         */
		public Cursor fetchDeviceTiles(long game_id) {
            Cursor c;
	    	
	    	String sql = "SELECT tile FROM deviceTiles " + 
	    	             "WHERE game_id=" + game_id;
	
	        c =  mDb.rawQuery(sql, null); 
	        return c;
		}
	    
	
	    
	    /**
	     * deletes a tile from the local player's shelf when played
	     * @param game_id index to which game
	     * @param tile - which tile # to delete
	     * @return 
	     */
	    public long  DeleteTile(long game_id, int tile) {
	    	return (long) mDb.delete(MYTILES_TABLE, KEY_TILE + "=" + tile + " AND " + KEY_GAME + "=" + game_id, null);
	    }
	    
	    /*
	     * same as DeleteTile but delete from device 
	     */
	    public long  DeleteDeviceTile(long game_id, int tile) {
	    	return (long) mDb.delete(DEVTILE_TABLE, KEY_TILE + "=" + tile + " AND " + KEY_GAME + "=" + game_id, null);
	    }
	    
	    /*
	     * same as DeleteTile but delete from localserver table 
	     */
	    public long  DeleteServerTile(long game_id, int tile) {
	    	return (long) mDb.delete(LSERVER_TABLE, KEY_TILE + "=" + tile + " AND " + KEY_GAME + "=" + game_id, null);
	    }
	    
	    /**
	     * 
	     * @param game_id game to delete
	     * @return
	     */
	    public long DeleteGame(long game_id ) {
	    	mDb.delete(CHATS_TABLE, KEY_GAME + "=" + game_id, null);
	    	mDb.delete(TILES_TABLE, KEY_ID + "=" + game_id, null);
	    	mDb.delete(MYTILES_TABLE, KEY_GAME + "=" + game_id, null);
	    	mDb.delete(DEVTILE_TABLE, KEY_GAME + "=" + game_id, null);
	    	mDb.delete(LSERVER_TABLE, KEY_GAME + "=" + game_id, null);
	    	return (long) mDb.delete(GAMES_TABLE, KEY_ID + "=" + game_id, null);
	    }

	    
	    /*
	     * deletes all games, tiles and username/id
	     */
	    public void DeleteUserData(){
	    	 long ret = mDb.delete(GAMES_TABLE, null, null);
	    	 ret = mDb.delete(TILES_TABLE, null, null);
	    	 ret = mDb.delete(MYTILES_TABLE, null, null);
	    	 ret = mDb.delete(PLAYER_TABLE, null, null);
	    	
	    }
	    
	    /**
	     * updates the local player's tiles with a new tile
	     * requires game_id, the tile and score if there is a score to be adjusted.
	     * @param game_id - which game
	     * @param tile - which tile to add
	     * @param score - current score
	     * @param adjustScore - a boolean to determine if we need to adjust the score or not
	     * @return 
	     */
	    
	    public Boolean UpdateTiles(long game_id, int tile, int score, Boolean adjustScore){
	    	int ret;
	    	Long retL;
	    	ContentValues args = new ContentValues();
	    	ContentValues args2 = new ContentValues();
	    	          // initially placing tiles into player's shelf from server is point-free
	    	          // however if, during play the player passed and couldn't place a tile
	    	          // then score is adjusted
	    	if (adjustScore) {
		    	
		        args.put(KEY_ID, game_id);
		        args.put(KEY_SCORE, score);
		        
		        ret = mDb.update(GAMES_TABLE, args, KEY_ID + "=" + game_id, null);
		        if (ret <= 0) {
		        	return false;
		        }
	    	}
	        args2.put(KEY_GAME, game_id);
	        args2.put(KEY_TILE, tile);
	        retL = mDb.insert(MYTILES_TABLE, null, args2);
	        if (retL <= 0) {
	        	return false;
	        }
	        return true;
	    	
	    }
	    
	    /*
	     * UpdateDeviceTiles does same as UpdateTiles except that the table is
	     * the deviceTiles and not mytiles. deviceTiles is used for when the user
	     * wants to play against the Android device
	     */
	    public Boolean UpdateDeviceTiles(long game_id, int tile, int score, Boolean adjustScore){
	    	int ret;
	    	Long retL;
	    	ContentValues args = new ContentValues();
	    	ContentValues args2 = new ContentValues();
	    	          // initially placing tiles into player's shelf from server is point-free
	    	          // however if, during play the player passed and couldn't place a tile
	    	          // then score is adjusted
	    	if (adjustScore) {
		    	
		        args.put(KEY_ID, game_id);
		        args.put(KEY_OPPSC, score);
		        
		        ret = mDb.update(GAMES_TABLE, args, KEY_ID + "=" + game_id, null);
		        if (ret <= 0) {
		        	return false;
		        }
	    	}
	        args2.put(KEY_GAME, game_id);
	        args2.put(KEY_TILE, tile);
	        retL = mDb.insert(DEVTILE_TABLE, null, args2);
	        if (retL <= 0) {
	        	return false;
	        }
	        return true;
	    	
	    }
	    
	    /*
	     * acting as local server, drawTile will pull one tile from the
	     * bone pile and insert it into the player's tile table designated by who
	     * the tile is then deleted from the localserver table
	     * returns true if a tile was available, false otherwise
	     */
	    public Boolean drawTile(long game_id, int who) {
            Cursor c;
            int tile = -1;
            int adj_score = -5;
            int tilesRemain = 0;
            Boolean ret;
            c = mDb.query(LSERVER_TABLE, new String[] {KEY_ID, KEY_GAME, KEY_TILE}, 
	                KEY_GAME+"="+game_id, null, null, null, null);
	    	
	        int old_score = getScore(game_id, who);
	        if (c.moveToLast()) {
	            tile = c.getInt(c.getColumnIndex(KEY_TILE));
	            tilesRemain = 1;
	            int new_score = old_score + adj_score;
	            
	            UpdateLastPlayed(game_id, getUserId(), who, new_score, 
	   				 -1,"t---",0,0,0,true, -5, tilesRemain, 0);  // got a tile, so decrement score and update tile
	            
	            if (who == 1) {                              // player is the device. Update device table
	            	 ret = UpdateDeviceTiles(game_id, tile, new_score, true);
	   		    }
	   		    else {
	   		         ret = UpdateTiles(game_id, tile, new_score, true);
	   		    }
	            
	            DeleteServerTile(game_id, tile);             // then delete from local "server"
	        }
	        else {
	        	tilesRemain = 0;
	        	ret = false;
	        	UpdateLastPlayed(game_id, getUserId(), who, old_score, 
   		    		 -1,"t---",0,0,0,false, 0, tilesRemain, 0);
	        }
	        return ret;
	    }
	    
	    public void UpdateLocalServerTiles(long game_id, int tile){
	    	int ret;
	    	Long retL;
	    	ContentValues args = new ContentValues();

		    args.put(KEY_GAME, game_id);
		    args.put(KEY_TILE, tile);

	        mDb.insert(LSERVER_TABLE, null, args);
	    }
	    	
	    
	    /**
	     *  create a new row in the games_table from challenging a remote user
	     * @param game_id - which game
	     * @param local_id - index in to local player's ID
	     * @param remote_id - index to server's index of remote players
	     * @param remote_name - string name of opponent
	     * @return  local user's ID, remote user's ID, and name the new game_id created by the server
	     *  
	     *  
	     */
	    public Long createNewGame(long game_id, int local_id, int remote_id, String username, String remote_name ){
	    	Long retval;
	    	ContentValues initialValues = new ContentValues();
	        initialValues.put(KEY_ID, game_id);
	        initialValues.put(KEY_USERID, local_id);
	        initialValues.put(KEY_NAME, username);
	        initialValues.put(KEY_SCORE, 0);
	        initialValues.put(KEY_OPP, remote_id);
	        initialValues.put(KEY_OPPNM, remote_name);
	        initialValues.put(KEY_OPPSC, 0);
	        initialValues.put(KEY_LASTT, -1);
	        initialValues.put(KEY_LASTN, "t000");
	        initialValues.put(KEY_ROW,0);
	        initialValues.put(KEY_COL,0);
	        initialValues.put(KEY_ROT,0);
	        initialValues.put(KEY_FACEDN,0);
	        initialValues.put(KEY_TSCORE, 0);
	        initialValues.put(KEY_WHO,remote_id); 
	        initialValues.put(KEY_REMAIN, 1);
	        initialValues.put(KEY_COMPLETE, 0);
	        int number = 0;
	        if (remote_id == -1) // solo game
	        	number = 56-9;
	        else
	        	number = 56-18; // 2 players 
	        initialValues.put(KEY_NUMREMAIN, number);
	        Calendar cal = Calendar.getInstance();
			//SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
			//String currentDateString =  sdf.format(cal.getTime());
	        initialValues.put(KEY_DATE, cal.getTimeInMillis());
	        
	        return mDb.insertOrThrow(GAMES_TABLE, null, initialValues);
	    	
	    }
	
	    /**
	     * 
	     * @param game_id - which game are we going to set to finished?
	     */
	    public void setGameOver( long game_id) {
	    	ContentValues args = new ContentValues();
	
	        args.put(KEY_COMPLETE, 1);
	        mDb.update(GAMES_TABLE, args, KEY_ID + "=" + game_id, null);    
	    }
	    
	   /**
	    * updateLastPlayed - updates the game with the last tile played
	    * @param game_id
	    * @param user_id - local user's id
	     * @param who - index to which player made the move
	     * @param score - score to update
	     * @param tile - tile number
	     * @param name - name of the tile, used for rebuilding faces
	     * @param row - row in the grid the tile was placed
	     * @param col - col in the grid the tile was placed
	     * @param rot - current rotation of the tile when placed
	     * @param facedn - orientation of the tile as placed
	     * @param tScore - score for just this turn
	     * @param remain - 0 = no remaining tiles, 1 = there are tiles left.
	     * @param complete - 0 = not complete, keep playing; 1=game over
	     * @return 
	    */
	    public void UpdateLastPlayed(long game_id, int user_id, int who, int score, int tile, String name,
	    		                     int row, int col, int rot, Boolean facedn, int tScore, int remain, int complete) {
	    	ContentValues args = new ContentValues();
	    	ContentValues InitialValues = new ContentValues();
	    	int newscore;
	    	
	    	args.put(KEY_ID, game_id);
	    	args.put(KEY_WHO, who);
	    	args.put(KEY_LASTT, tile);
	        args.put(KEY_LASTN, name);
	        args.put(KEY_ROW, row);
	        args.put(KEY_COL, col);
	        args.put(KEY_ROT, rot);
	        int face = (facedn)? 1 :0;
	        args.put(KEY_FACEDN, face);
	    	args.put(KEY_TSCORE, tScore);
	    	args.put(KEY_REMAIN, remain);
	
	        Calendar cal = Calendar.getInstance();
	        args.put(KEY_DATE, cal.getTimeInMillis());
	        int tilesRemaining = getNumRemaining(game_id);
	        if (user_id == who) {   // last player was me
	        	if (tScore == -5) { 
	            	newscore = score + getScore(game_id, user_id); // adjust myscore
	            	args.put(KEY_SCORE, newscore);
	            	args.put(KEY_LASTT, -1);
	            	tilesRemaining --;   // this player took a tile
	            }
	        	else 
	        		args.put(KEY_SCORE, score);
	        }
	        else {                 // last player was opponent
	        	if (tScore == -5) { 
	            	newscore = score + getScore(game_id, who); // adjust opponent's
	        	    args.put(KEY_OPPSC, newscore);
	            	args.put(KEY_LASTT, -1);
	            	tilesRemaining --;   // opponent took a tile
	
	        	}
	        	else
	        	    args.put(KEY_OPPSC, score);
	
	        }
	       
	    	args.put(KEY_NUMREMAIN, tilesRemaining);
	    	mDb.update(GAMES_TABLE, args, KEY_ID + "=" + game_id, null);    // this table goes to 
	    	                                                                // and comes from the main server
	  	
	    	InitialValues.put(KEY_ID, game_id);
	    	InitialValues.put(KEY_ROW, row);
	    	InitialValues.put(KEY_COL, col);
	    	InitialValues.put(KEY_ROT, rot);
	    	InitialValues.put(KEY_TILE, tile);
	    	InitialValues.put(KEY_FACEDN, face);
	    	mDb.insert(TILES_TABLE, null, InitialValues);                   // this table is local only 
	        
	    }
	    
	    public Cursor fetchLastPlayed(long game_id){
	        Cursor c;
	        return mDb.query(GAMES_TABLE, new String[] {KEY_ID, KEY_SCORE,
	        KEY_OPP, KEY_OPPSC, KEY_OPPNM, KEY_DATE, KEY_LASTT, KEY_LASTN, KEY_TSCORE, KEY_WHO}, 
	        KEY_ID + "=" + game_id, null, null, null, KEY_DATE);
	
	    }

	  
	}
