package com.ulsanonline.triominoes;

/*
 * 02/22/2012 - v1.001 Initial release
 * 03/17/2012 - v3.0   Added leaderboard stats
 * 07/15/2012 - v3.2   check for network availability
 * 09/29/2013 - v4.3   adding needed threading for network traffic method calls
 * 
 */ 

import java.util.Calendar;
import android.content.Context;
import android.app.Activity;
import android.app.AlarmManager;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.app.PendingIntent;
import android.database.Cursor;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.os.StrictMode;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewSwitcher;
import org.json.JSONException;


public class triominoes extends Activity {

	private EditText mNameText;  // username
	private TextView mWelcome; 
	private String userName;
	private String player2name;
	private int userId;
	private Context mCtx;
    private ServerCommunication sComm;
    private Boolean good_to_go;
    private triDbAdapter mDbHelper;
	private ViewSwitcher mViewSwitcher;
	private TextView mTextField;
	private PendingIntent pendingIntent;
	private Settings s;
	private int user_id;	    
	private ProgressDialog dialogUI; 
	
	


	@Override
	protected void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);
		mCtx = this;
		mDbHelper = new triDbAdapter(this.getApplicationContext() );
		mDbHelper.open();
        s = new Settings(mCtx);
		if(!isNetworkAvailable()) {
		    // can't find a network. Nothing's gonna work
			Toast.makeText(mCtx, "Network Unavailable. Check Settings", Toast.LENGTH_SHORT).show();
		}
		
		if (android.os.Build.VERSION.SDK_INT > 9) {
		    StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
		    StrictMode.setThreadPolicy(policy);
		}
		
		dialogUI = new ProgressDialog(this);
		dialogUI.setMessage("Communicating...");  // I'll use this several times in the game
		

		sComm = new ServerCommunication(this.getApplicationContext());
		good_to_go = false;
		Cursor c = mDbHelper.fetchPlayer();
		if (c.moveToLast()) {
		   int nameColumn = c.getColumnIndex(mDbHelper.KEY_NAME);
		   userName = c.getString(nameColumn);
		   userId = c.getInt(c.getColumnIndex(mDbHelper.KEY_ID));
		   good_to_go = true;
		}
		c.close();
		
		setContentView(R.layout.initial_form);
		
		/*
		Bundle bundle = new Bundle();
    	bundle.putInt(triDbAdapter.KEY_ID, 824);
		Intent i = new Intent(this, TriOminoesStartActivity.class);
		i.putExtras(bundle); 
		startActivity(i);
		*/
		
		
		mViewSwitcher = (ViewSwitcher) findViewById(R.id.viewswitcher);
		mNameText = (EditText) findViewById(R.id.username);
		Button submitButton = (Button) findViewById(R.id.submit);
        submitButton.setOnClickListener(doIt);
        mWelcome = (TextView) findViewById(R.id.welcome);
        
        Button resetButton = (Button) findViewById(R.id.reset);
        resetButton.setOnClickListener(Reset);
        
		if (good_to_go) {
			mWelcome.setText("Welcome, "+mDbHelper.getName());
			// read settings from dB
			mDbHelper.getSettings();
			// setup timers
			
			if (Settings.AlarmInterval != 0) {
				Calendar calendar = Calendar.getInstance();
		        calendar.setTimeInMillis(System.currentTimeMillis());
		        calendar.add(Calendar.MINUTE, (int)( Settings.AlarmInterval/1000/60));
		        
		        Intent myIntent = new Intent(mCtx, WakeCheck.class);
		
		        pendingIntent = PendingIntent.getService(mCtx, 0, myIntent, 0);
		
		        AlarmManager alarmManager = (AlarmManager)mCtx.getSystemService(mCtx.ALARM_SERVICE);
		        alarmManager.cancel(pendingIntent); // cancel any previous alarms
		        // InExactrepeating works - every fucking minute it wakes up and ignore the interval
		        //alarmManager.setInexactRepeating(AlarmManager.RTC_WAKEUP,  calendar.getTimeInMillis(), Settings.AlarmInterval, pendingIntent);
		        alarmManager.setRepeating(AlarmManager.RTC_WAKEUP,  calendar.getTimeInMillis(), Settings.AlarmInterval, pendingIntent);
			}

			
			
			
		}
		else { 
			mViewSwitcher.showNext();
			setTitle("Tri-Ominoes: New User Signup");
		}

	}
	
	public boolean isNetworkAvailable() {
	    ConnectivityManager cm = (ConnectivityManager) 
	    	getSystemService(Context.CONNECTIVITY_SERVICE);
	    NetworkInfo networkInfo = cm.getActiveNetworkInfo();
	    // if no network is available networkInfo will be null
	    // otherwise check if we are connected
	    if (networkInfo != null && networkInfo.isConnected()) {
	        return true;
	    }
	    return false;
	} 

	DialogInterface.OnClickListener dialogClickListener = new DialogInterface.OnClickListener() {
	    public void onClick(DialogInterface dialog, int which) {
	        switch (which){
	        case DialogInterface.BUTTON_POSITIVE:
	            //Yes button clicked - delete all data
	        	user_id = mDbHelper.getUserId();
	        	if (user_id >= 0) {   // delete from server, too.       		
		        	
				    Thread deletedude = new Thread() {  
				        public void run() {

			   		    	Looper.prepare();
			   		    	sComm.deleteUser(user_id);
							
					    	handler.sendEmptyMessage(0);
					    }
					};
					deletedude.start();
	        	   
	        	}
	        	mDbHelper.DeleteUserData();
	        	good_to_go = false;
	        	mViewSwitcher.showNext();
				setTitle("Tri-Ominoes: New User Signup");
	            break;

	        case DialogInterface.BUTTON_NEGATIVE:
	            //No button clicked - do nothing
	            break;
	        }
	    }
	};
	
	// when thread has finished, just kill the dialog
	final Handler handler = new Handler() {
	    public void handleMessage(Message msg) {
			dialogUI.dismiss();
		}
	};


	// Will be called via the onClick attribute
	// of the buttons in initial_form.xml
	public void onClick(View view) {
		//@SuppressWarnings("unchecked")
		Intent i = null;
	
		switch (view.getId()) {
		case R.id.help:
			i = new Intent(mCtx,Help.class);
			break;
			
		case R.id.settings:
			i = new Intent(mCtx,ChangeSettings.class);
			break;
			
		case R.id.begin:
			i = new Intent(mCtx,SelectGames.class);
			break;
			
		case R.id.stats:
			i = new Intent(Intent.ACTION_VIEW);
	        i.setData(Uri.parse("http://martypants.us/triominoes"));
	        break;
		}
		startActivity(i);
	}
	
	private OnClickListener Reset = new OnClickListener() {
		public void onClick(View view) {
			
			AlertDialog.Builder builder = new AlertDialog.Builder(mCtx);
			builder.setMessage("Are you sure you want to delete ALL games and User ID?").setPositiveButton("Accept", dialogClickListener)
			    .setNegativeButton("Decline", dialogClickListener).show();
	   }
	};
	
    private OnClickListener doIt = new OnClickListener() {
        public void onClick(View v) {
            InputMethodManager manager = (InputMethodManager)
            getSystemService(Context.INPUT_METHOD_SERVICE);
            IBinder binder = v.getApplicationWindowToken();
            if (binder != null) {
                manager.hideSoftInputFromWindow(binder, 0);
            } 
        	userName = mNameText.getText().toString();
        	if (userName.isEmpty()) {
        		Toast.makeText(mCtx, "Um...you need a name here.", Toast.LENGTH_SHORT).show();
        	}
        	else {
    			boolean error = false;
    			String msg = "";
    			
    			
    			new AsyncTask<Void,Void, Boolean>(){
    				boolean goodName = false;
    				String msg = "";
    				
                    @Override
                    protected void onPreExecute() {
                    	dialogUI.show();
                    }
                    
                    @Override
    				protected Boolean doInBackground(Void... params) {
                    	try {
							goodName = sComm.registerUser(userName);
						} catch (JSONException e) {
							msg += "ERROR: Comunication:..."+e.toString();
							goodName = false;
							e.printStackTrace();
						}
                    	if (goodName) 
                    		msg += "Congratulations! You're ready to play Tri-Ominoes";
                    	else
                    		msg += "ERROR: Sorry, username:" + userName +" already exists. Try another name.";
						return goodName;
                    }
                    
                    @Override
                    protected void onPostExecute(Boolean error) {
                    	dialogUI.dismiss();
						Toast.makeText(mCtx, msg, Toast.LENGTH_SHORT).show();

                    	if (!error) {
    						good_to_go = true;
    						mViewSwitcher.showNext();
    						mWelcome.setText("Welcome, "+mDbHelper.getName());
    					}
                    	
                    }
                    
    			}.execute();
    		

				
        	}
        	
        }

	};
	
	@Override
	public void onDestroy() {
		super.onDestroy();
		mDbHelper.close();
	}

}
